# /// script
# requires-python = ">=3.10,<4.0"
# dependencies = [
#     "datasets==3.5.0",
#     "huggingface-hub==0.24.0",
#     "pandas==2.2.0",
#     "requests==2.32.3",
#     "tqdm==4.67.1",
# ]
# ///

"""Create the ScaLA datasets and upload them to the HF Hub."""

import random
import re
import warnings

import pandas as pd
from datasets.arrow_dataset import Dataset
from datasets.dataset_dict import DatasetDict
from huggingface_hub.hf_api import HfApi
from pandas.errors import SettingWithCopyWarning
from tqdm.auto import tqdm

from .constants import MAX_NUM_CHARS_IN_DOCUMENT, MIN_NUM_CHARS_IN_DOCUMENT  # noqa
from .load_ud_pos import (
    load_bgdt_pos,
    load_cadt_pos,
    load_csdt_pos,
    load_dadt_pos,
    load_dedt_pos,
    load_eldt_pos,
    load_endt_pos,
    load_esdt_pos,
    load_etdt_pos,
    load_fidt_pos,
    load_fodt_pos,
    load_frdt_pos,
    load_hrdt_pos,
    load_hudt_pos,
    load_isdt_pos,
    load_itdt_pos,
    load_ltdt_pos,
    load_lvdt_pos,
    load_nldt_pos,
    load_nodt_nb_pos,
    load_nodt_nn_pos,
    load_pldt_pos,
    load_ptdt_pos,
    load_rodt_pos,
    load_skdt_pos,
    load_sldt_pos,
    load_sqdt_pos,
    load_srdt_pos,
    load_svdt_pos,
    load_ukdt_pos,
)


def main() -> None:
    """Create the ScaLA datasets and upload them to the HF Hub."""
    # Set up the POS dataset loaders
    pos_datasets = {
        "da": load_dadt_pos,
        "nb": load_nodt_nb_pos,
        "nn": load_nodt_nn_pos,
        "sv": load_svdt_pos,
        "is": load_isdt_pos,
        "fo": load_fodt_pos,
        "de": load_dedt_pos,
        "nl": load_nldt_pos,
        "en": load_endt_pos,
        "fr": load_frdt_pos,
        "it": load_itdt_pos,
        "es": load_esdt_pos,
        "fi": load_fidt_pos,
        "pt": load_ptdt_pos,
        "lv": load_lvdt_pos,
        "et": load_etdt_pos,
        "pl": load_pldt_pos,
        "lt": load_ltdt_pos,
        "cs": load_csdt_pos,
        "sk": load_skdt_pos,
        "uk": load_ukdt_pos,
        "el": load_eldt_pos,
        "bg": load_bgdt_pos,
        "sr": load_srdt_pos,
        "sl": load_sldt_pos,
        "hr": load_hrdt_pos,
        "hu": load_hudt_pos,
        "ro": load_rodt_pos,
        "ca": load_cadt_pos,
        "sq": load_sqdt_pos,
    }

    # Set up the progress bar and iterate over the languages
    with tqdm(pos_datasets.items(), desc="Creating ScaLA datasets") as pbar:
        for lang, fn in pbar:
            # Update the progress bar description
            pbar.set_description(f"Creating ScaLA datasets - {lang}")

            # Load the POS dataset
            pos_dataset = fn()

            # Merge the DDT POS dataframes to a single dataframe, with columns `ids`,
            # `tokens`, `doc` and `pos_tags`
            df = pd.concat(pos_dataset.values(), ignore_index=True)
            assert isinstance(df, pd.DataFrame)

            # Drop the duplicates
            df = df.drop_duplicates(subset="doc").reset_index(drop=True)
            assert isinstance(df, pd.DataFrame)

            # Remove samples with five or fewer tokens
            df = df[df.tokens.map(lambda lst: len(lst) > 5)]
            assert isinstance(df, pd.DataFrame)

            # Remove samples with five or fewer distinct POS tags
            df = df[df.pos_tags.map(lambda lst: len(set(lst)) > 5)]
            assert isinstance(df, pd.DataFrame)

            # Remove samples with an odd number of quotes
            df = df[df.doc.map(lambda doc: doc.count('"') % 2 == 0)]
            assert isinstance(df, pd.DataFrame)

            # Remove samples which starts with punctuation
            df = df[df.pos_tags.map(lambda lst: lst[0] not in ["PUNCT", "SYM"])]
            assert isinstance(df, pd.DataFrame)

            # Remove samples containing more than one '=' character, as this is used to
            # indicate a tag
            df = df[df.doc.map(lambda doc: doc.count("=") <= 1)]
            assert isinstance(df, pd.DataFrame)

            # Remove samples containing 'SLUTORD', as this is used to indicate a tag
            df = df[~df.doc.str.contains("SLUTORD")]
            assert isinstance(df, pd.DataFrame)

            # Create a training, validation, test and small training set. Note that we
            # will corrupt the data, so this is only half the size of the final
            # datasets. In the case where the dataframe does not contain enough samples
            # for all the splits, we keep halving the test size until we have enough
            # samples.
            test_size = 1024
            while test_size >= 128:
                try:
                    train_df, val_df, test_df, full_train_df = make_splits(
                        df=df, train_size=512, val_size=128, test_size=test_size
                    )
                    break
                except ValueError:
                    test_size //= 2
            else:
                train_size = 64
                val_size = 32
                test_size = len(df) - train_size - val_size
                assert test_size > 0, "Not enough samples to create the splits."
                train_df, val_df, test_df, full_train_df = make_splits(
                    df=df, train_size=train_size, val_size=val_size, test_size=test_size
                )

            assert isinstance(train_df, pd.DataFrame)
            assert isinstance(val_df, pd.DataFrame)
            assert isinstance(test_df, pd.DataFrame)
            assert isinstance(full_train_df, pd.DataFrame)

            # Only work with samples where the document is not very large or small We do
            # it after we have made the splits to ensure that the dataset is minimally
            # affected.
            new_train_df = train_df.copy()
            new_train_df["text_len"] = new_train_df.doc.str.len()
            new_train_df = new_train_df.query(
                "text_len >= @MIN_NUM_CHARS_IN_DOCUMENT"
            ).query("text_len <= @MAX_NUM_CHARS_IN_DOCUMENT")
            new_val_df = val_df.copy()
            new_val_df["text_len"] = new_val_df.doc.str.len()
            new_val_df = new_val_df.query(
                "text_len >= @MIN_NUM_CHARS_IN_DOCUMENT"
            ).query("text_len <= @MAX_NUM_CHARS_IN_DOCUMENT")
            new_test_df = test_df.copy()
            new_test_df["text_len"] = new_test_df.doc.str.len()
            new_test_df = new_test_df.query(
                "text_len >= @MIN_NUM_CHARS_IN_DOCUMENT"
            ).query("text_len <= @MAX_NUM_CHARS_IN_DOCUMENT")
            new_full_train_df = full_train_df.copy()
            new_full_train_df["text_len"] = new_full_train_df.doc.str.len()
            new_full_train_df = new_full_train_df.query(
                "text_len >= @MIN_NUM_CHARS_IN_DOCUMENT"
            ).query("text_len <= @MAX_NUM_CHARS_IN_DOCUMENT")

            # Add the corrupted data and turn the dataframes into Hugging Face Dataset
            # objects
            train = prepare_df(new_train_df, split="train")
            val = prepare_df(new_val_df, split="val")
            test = prepare_df(new_test_df, split="test")
            full_train = prepare_df(new_full_train_df, split="train")

            # Collect datasets in a dataset dictionary
            dataset = DatasetDict(
                {"train": train, "val": val, "test": test, "full_train": full_train}
            )

            # Push the dataset to the Hugging Face Hub
            dataset_id = f"EuroEval/scala-{lang}"
            HfApi().delete_repo(dataset_id, repo_type="dataset", missing_ok=True)
            dataset.push_to_hub(dataset_id, private=True)


def join_tokens(tokens: list[str]) -> str:
    """Joins a list of tokens into a string.

    Args:
        tokens:
            The list of tokens to join.

    Returns:
        The joined string.
    """
    # Form document
    doc = " ".join(tokens)

    # Remove whitespace around punctuation
    doc = (
        doc.replace(" .", ".")
        .replace(" ,", ",")
        .replace(" ;", ";")
        .replace(" :", ":")
        .replace("( ", "(")
        .replace(" )", ")")
        .replace("[ ", "[")
        .replace(" ]", "]")
        .replace("{ ", "{")
        .replace(" }", "}")
        .replace(" ?", "?")
        .replace(" !", "!")
    )

    # Remove whitespace around quotes
    if doc.count('"') % 2 == 0:
        doc = re.sub('" ([^"]*) "', '"\\1"', doc)

    # Return the document
    return doc


def delete(tokens: list[str], pos_tags: list[str]) -> str | None:
    """Delete a random token from a list of tokens.

    The POS tags are used to prevent deletion of a token which does not make the
    resulting sentence grammatically incorrect, such as removing an adjective or an
    adverb.

    Args:
        tokens:
            The list of tokens to delete from.
        pos_tags:
            The list of POS tags for the tokens.

    Returns:
        The deleted token, or None if no token could be deleted.
    """
    # Copy the token list
    new_tokens = tokens.copy()

    # Get candidate indices to remove. We do not remove adjectives, adverbs,
    # punctuation, determiners or numbers, as the resulting sentence will probably
    # still be grammatically correct. Further, we do not remove nouns or proper nouns
    # if they have another noun or proper noun as neighbour, as that usually does not
    # make the sentence incorrect either.
    indices = [
        idx
        for idx, pos_tag in enumerate(pos_tags)
        if pos_tag not in ["ADJ", "ADV", "PUNCT", "SYM", "DET", "NUM", "PART"]
        and (
            pos_tag not in ["NOUN", "PROPN"]
            or (
                (idx == 0 or pos_tags[idx - 1] not in ["NOUN", "PROPN"])
                and (
                    idx == len(new_tokens) - 1
                    or pos_tags[idx + 1] not in ["NOUN", "PROPN"]
                )
            )
        )
    ]

    # If there are no candidates then return None
    if len(indices) == 0:
        return None

    # Get the random index
    rnd_idx = random.choice(indices)

    # Delete the token at the index
    new_tokens.pop(rnd_idx)

    # Join up the new tokens and return the string
    return join_tokens(new_tokens)


def flip_neighbours(tokens: list[str], pos_tags: list[str]) -> str | None:
    """Flip a pair of neighbouring tokens.

    The POS tags are used to prevent flipping of tokens which does not make the
    resulting sentence grammatically incorrect, such as flipping two adjectives.

    Args:
        tokens:
            The list of tokens to flip.
        pos_tags:
            The list of POS tags for the tokens.

    Returns:
        The flipped string, or None if no flip was possible.
    """
    # Copy the token list
    new_tokens = tokens.copy()

    # Collect all indices that are proper words, and which has a neighbour which is
    # also a proper word as well as having a different POS tag
    indices = [
        idx for idx, pos_tag in enumerate(pos_tags) if pos_tag not in ["PUNCT", "SYM"]
    ]
    indices = [
        idx
        for idx in indices
        if (idx + 1 in indices and pos_tags[idx] != pos_tags[idx + 1])
        or (idx - 1 in indices and pos_tags[idx] != pos_tags[idx - 1])
    ]

    # If there are fewer than two relevant tokens then return None
    if len(indices) < 2:
        return None

    # Get the first random index
    rnd_fst_idx = random.choice(indices)

    # Get the second (neighbouring) index
    if rnd_fst_idx == 0:
        rnd_snd_idx = rnd_fst_idx + 1
    elif rnd_fst_idx == len(tokens) - 1:
        rnd_snd_idx = rnd_fst_idx - 1
    elif (
        pos_tags[rnd_fst_idx + 1] in ["PUNCT", "SYM"]
        or pos_tags[rnd_fst_idx] == pos_tags[rnd_fst_idx + 1]
        or {pos_tags[rnd_fst_idx], pos_tags[rnd_fst_idx + 1]} == {"PRON", "AUX"}
    ):
        rnd_snd_idx = rnd_fst_idx - 1
    elif (
        pos_tags[rnd_fst_idx - 1] in ["PUNCT", "SYM"]
        or pos_tags[rnd_fst_idx] == pos_tags[rnd_fst_idx - 1]
        or {pos_tags[rnd_fst_idx], pos_tags[rnd_fst_idx + 1]} == {"PRON", "AUX"}
    ):
        rnd_snd_idx = rnd_fst_idx + 1
    elif random.random() > 0.5:
        rnd_snd_idx = rnd_fst_idx - 1
    else:
        rnd_snd_idx = rnd_fst_idx + 1

    # Flip the two indices
    new_tokens[rnd_fst_idx] = tokens[rnd_snd_idx]
    new_tokens[rnd_snd_idx] = tokens[rnd_fst_idx]

    # If we flipped the first character, then ensure that the new first character is
    # title-cased and the second character is of lower case. We only do this if they
    # are not upper cased, however.
    if rnd_fst_idx == 0 or rnd_snd_idx == 0:
        if new_tokens[0] != new_tokens[0].upper():
            new_tokens[0] = new_tokens[0].title()
        if new_tokens[1] != new_tokens[1].upper():
            new_tokens[1] = new_tokens[1].lower()

    # Join up the new tokens and return the string
    return join_tokens(new_tokens)


def corrupt(
    tokens: list[str], pos_tags: list[str], num_corruptions: int = 1
) -> list[tuple[str, str]]:
    """Corrupt a list of tokens.

    This randomly either flips two neighbouring tokens or deletes a random token.

    Args:
        tokens:
            The list of tokens to corrupt.
        pos_tags:
            The list of POS tags for the tokens.
        num_corruptions:
            The number of corruptions to perform. Defaults to 1.

    Returns:
        The list of (corrupted_string, corruption_type)
    """
    # Define the list of corruptions
    corruptions: list[tuple[str, str]] = list()

    # Continue until we have achieved the desired number of corruptions
    while len(corruptions) < num_corruptions:
        # Choose which corruption to perform, at random
        corruption_fn = random.choice([flip_neighbours, delete])

        # Corrupt the tokens
        corruption = corruption_fn(tokens, pos_tags)

        # If the corruption succeeded, and that we haven't already performed the same
        # corruption, then add the corruption to the list of corruptions
        if corruption not in corruptions and corruption is not None:
            corruptions.append((corruption, corruption_fn.__name__))

    # Return the list of corruptions
    return corruptions


def prepare_df(df: pd.DataFrame, split: str) -> Dataset:
    """Prepare a dataframe by adding an equal number of corruptions to it.

    Args:
        df:
            The dataframe to prepare.
        split:
            The split to prepare the dataframe for.

    Returns:
        The prepared dataset.
    """
    # Reset the index of the dataframe
    df.reset_index(drop=True, inplace=True)

    # Get the corrupted strings
    corrupted_list = [
        corrupt(tokens=tokens, pos_tags=pos_tags, num_corruptions=1)[0]
        for tokens, pos_tags in zip(df.tokens, df.pos_tags)
    ]

    # Add the corrupted strings to the dataframe
    with warnings.catch_warnings():
        warnings.simplefilter("ignore", category=SettingWithCopyWarning)
        df["corrupted"] = [tup[0] for tup in corrupted_list]
        df["corruption_type"] = [tup[1] for tup in corrupted_list]

    # Restructure the dataframe to have columns `text`, `corruption_type` and `label`,
    # with one sample per row
    df = pd.concat(
        [
            pd.DataFrame(
                dict(
                    text=df.tokens.map(join_tokens).tolist(),
                    corruption_type=[None for _ in range(len(df))],
                    label=["correct" for _ in range(len(df))],
                )
            ),
            pd.DataFrame(
                dict(
                    text=df.corrupted.explode().tolist(),
                    corruption_type=df.corruption_type.explode().tolist(),
                    label=["incorrect" for _ in range(len(df))],
                )
            ),
        ]
    )

    # Shuffle the dataframe
    df = df.sample(frac=1.0, random_state=4242).reset_index(drop=True)

    # Convert the dataframe to a Hugging Face Dataset and return it
    return Dataset.from_pandas(df, split=split)  # type: ignore[bad-argument-type]


def make_splits(
    df: pd.DataFrame, train_size: int, val_size: int, test_size: int
) -> tuple[pd.DataFrame, pd.DataFrame, pd.DataFrame, pd.DataFrame]:
    """Make the splits.

    Args:
        df:
            The dataframe to make the splits from.
        train_size:
            The size of the training split.
        val_size:
            The size of the validation split.
        test_size:
            The size of the test split.

    Returns:
        The train, validation, test, and full training splits.
    """
    val_df = df.sample(n=val_size, random_state=4242)
    df_filtered = df[~df.index.isin(val_df.index)]
    test_df = df_filtered.sample(n=test_size, random_state=4242)
    full_train_df = df_filtered[~df_filtered.index.isin(test_df.index)]
    assert isinstance(full_train_df, pd.DataFrame)
    train_df = full_train_df.sample(n=train_size, random_state=4242)
    return train_df, val_df, test_df, full_train_df


if __name__ == "__main__":
    main()



# /// script
# requires-python = ">=3.10,<4.0"
# dependencies = [
#     "datasets==3.5.0",
#     "huggingface-hub==0.24.0",
#     "pandas==2.2.0",
#     "requests==2.32.3",
# ]
# ///

"""Create the DACSA summarization datasets and upload to HF Hub."""

import pandas as pd
from datasets import Dataset, DatasetDict, Split, load_dataset
from huggingface_hub import HfApi

from .constants import MAX_NUM_CHARS_IN_ARTICLE, MIN_NUM_CHARS_IN_ARTICLE


def main() -> None:
    """Create the DACSA summarization datasets and upload to HF Hub."""
    # Load the DACSA dataset
    dacsa_id = "ELiRF/dacsa"
    languages = {"ca": "catalan", "es": "spanish"}

    for lang, subset_name in languages.items():
        dataset = load_dataset(dacsa_id, subset_name)
        assert isinstance(dataset, DatasetDict)

        train_df = dataset["train"].to_pandas()
        val_df = dataset["validation"].to_pandas()
        test_df = dataset["test.ni"].to_pandas()
        assert isinstance(train_df, pd.DataFrame)
        assert isinstance(val_df, pd.DataFrame)
        assert isinstance(test_df, pd.DataFrame)

        train_df = process_df(df=train_df)
        val_df = process_df(df=val_df)
        test_df = process_df(df=test_df)

        train_df_final, val_df_final, test_df_final = make_splits(
            train_df=train_df, val_df=val_df, test_df=test_df
        )

        # Collect datasets in a dataset dictionary
        mini_dataset = DatasetDict(
            {
                "train": Dataset.from_pandas(train_df_final, split=Split.TRAIN),
                "val": Dataset.from_pandas(val_df_final, split=Split.VALIDATION),
                "test": Dataset.from_pandas(test_df_final, split=Split.TEST),
            }
        )

        mini_dataset_id = f"EuroEval/dacsa-{lang}-mini"
        HfApi().delete_repo(mini_dataset_id, repo_type="dataset", missing_ok=True)
        mini_dataset.push_to_hub(mini_dataset_id, private=True)


def process_df(df: pd.DataFrame) -> pd.DataFrame:
    """Process the dataframe.

    Args:
        df: A dataframe to process.

    Returns:
        Processed dataframe.
    """
    # Renames
    df = df.rename(columns={"article": "text", "summary": "target_text"})

    # Keep only samples where the text is not very large or small
    lengths = df.text.str.len()
    lower_bound = MIN_NUM_CHARS_IN_ARTICLE
    upper_bound = MAX_NUM_CHARS_IN_ARTICLE
    df = df.loc[lengths.between(lower_bound, upper_bound)]

    # Keep only the necessary columns
    keep_columns = ["text", "target_text"]
    df = df.loc[keep_columns]
    return df


def make_splits(
    train_df: pd.DataFrame, val_df: pd.DataFrame, test_df: pd.DataFrame
) -> tuple[pd.DataFrame, pd.DataFrame, pd.DataFrame]:
    """Make the splits.

    Args:
        train_df: A dataframe to make the train split from.
        val_df: A dataframe to make the validation split from.
        test_df: A dataframe to make the test split from.

    Returns:
        The train, validation, and test splits.
    """
    # Create train split
    train_size = 1024
    train_df_final = train_df.sample(n=train_size, random_state=4242)

    # Create validation split
    val_size = 256
    val_df_final = val_df.sample(n=val_size, random_state=4242)

    # Create test split
    test_size = 2048
    test_df_final = test_df.sample(n=test_size, random_state=4242)

    # Reset the index
    train_df_final = train_df_final.reset_index(drop=True)
    val_df_final = val_df_final.reset_index(drop=True)
    test_df_final = test_df_final.reset_index(drop=True)

    return train_df_final, val_df_final, test_df_final


if __name__ == "__main__":
    main()



# /// script
# requires-python = ">=3.10,<4.0"
# dependencies = [
#     "datasets==3.5.0",
#     "huggingface-hub==0.24.0",
#     "pandas==2.2.0",
#     "requests==2.32.3",
#     "scikit-learn<1.6.0",
# ]
# ///

"""Create the LT-History knowledge dataset and upload it to the HF Hub."""

from collections import Counter

import pandas as pd
import requests
from datasets import Dataset, DatasetDict, Split
from huggingface_hub import HfApi

from .constants import (
    CHOICES_MAPPING,
    MAX_NUM_CHARS_IN_INSTRUCTION,
    MAX_NUM_CHARS_IN_OPTION,
    MAX_REPETITIONS,
    MIN_NUM_CHARS_IN_INSTRUCTION,
    MIN_NUM_CHARS_IN_OPTION,
)


def main() -> None:
    """Create the Lithuanian History knowledge dataset and upload it to the HF Hub."""
    # Download the JSON data from GitHub
    url = (
        "https://raw.githubusercontent.com/OpenBabylon/NoDaLiDa2025-LT-History-Eval"
        "/refs/heads/main/lit_data.json"
    )
    response = requests.get(url)
    response.raise_for_status()
    data = response.json()

    # Convert to DataFrame
    rows = []
    for item in data:
        # Extract question stem
        question_stem = item["question"]["stem"]

        # Extract choices
        choices = item["question"]["choices"]
        choice_texts = {choice["label"]: choice["text"] for choice in choices}

        # Get answer key and convert to lowercase
        answer_key = item["answerKey"].lower()

        # Create row
        row = {
            "id": item["id"],
            "instruction": question_stem,
            "option_a": choice_texts.get("A", ""),
            "option_b": choice_texts.get("B", ""),
            "option_c": choice_texts.get("C", ""),
            "option_d": choice_texts.get("D", ""),
            "label": answer_key,
        }
        rows.append(row)

    df = pd.DataFrame(rows)

    # Remove samples with null values in any option columns
    df = df[
        df["option_a"].notnull()
        & df["option_b"].notnull()
        & df["option_c"].notnull()
        & df["option_d"].notnull()
    ]

    # Remove samples with overly short or long texts
    df = df[
        (df.instruction.str.len() >= MIN_NUM_CHARS_IN_INSTRUCTION)
        & (df.instruction.str.len() <= MAX_NUM_CHARS_IN_INSTRUCTION)
        & (df.option_a.str.len() >= MIN_NUM_CHARS_IN_OPTION)
        & (df.option_a.str.len() <= MAX_NUM_CHARS_IN_OPTION)
        & (df.option_b.str.len() >= MIN_NUM_CHARS_IN_OPTION)
        & (df.option_b.str.len() <= MAX_NUM_CHARS_IN_OPTION)
        & (df.option_c.str.len() >= MIN_NUM_CHARS_IN_OPTION)
        & (df.option_c.str.len() <= MAX_NUM_CHARS_IN_OPTION)
        & (df.option_d.str.len() >= MIN_NUM_CHARS_IN_OPTION)
        & (df.option_d.str.len() <= MAX_NUM_CHARS_IN_OPTION)
    ]
    assert isinstance(df, pd.DataFrame)

    def is_repetitive(text: str) -> bool:
        """Return True if the text is repetitive."""
        max_repetitions = max(Counter(text.split()).values())
        return max_repetitions > MAX_REPETITIONS

    # Remove overly repetitive samples
    df = df[
        ~df.instruction.apply(is_repetitive)
        & ~df.option_a.apply(is_repetitive)
        & ~df.option_b.apply(is_repetitive)
        & ~df.option_c.apply(is_repetitive)
        & ~df.option_d.apply(is_repetitive)
    ]

    # Create the `text` column with all options formatted
    df["text"] = [
        row.instruction.replace("\n", " ").strip() + "\n"
        f"{CHOICES_MAPPING['lt']}:\n"
        "a. " + row.option_a.replace("\n", " ").strip() + "\n"
        "b. " + row.option_b.replace("\n", " ").strip() + "\n"
        "c. " + row.option_c.replace("\n", " ").strip() + "\n"
        "d. " + row.option_d.replace("\n", " ").strip()
        for _, row in df.iterrows()
    ]

    # Keep only the required columns for EuroEval format
    df = df[["text", "label"]]
    assert isinstance(df, pd.DataFrame)

    # Remove duplicates
    df.drop_duplicates(inplace=True)
    df.reset_index(drop=True, inplace=True)

    # Create train and test splits
    train_size = 64
    val_size = 32
    train_df = df.sample(n=train_size, random_state=42)
    df.drop(index=train_df.index.tolist(), inplace=True)
    val_df = df.sample(n=val_size, random_state=42)
    test_df = df.drop(index=val_df.index.tolist())

    # Reset the index
    train_df = train_df.reset_index(drop=True)
    val_df = val_df.reset_index(drop=True)
    test_df = test_df.reset_index(drop=True)

    # Collect datasets in a dataset dictionary (no train split)
    dataset = DatasetDict(
        {
            "train": Dataset.from_pandas(train_df, split=Split.TRAIN),
            "val": Dataset.from_pandas(val_df, split=Split.VALIDATION),
            "test": Dataset.from_pandas(test_df, split=Split.TEST),
        }
    )

    # Create dataset ID
    dataset_id = "EuroEval/lt-history"

    # Remove the dataset from Hugging Face Hub if it already exists
    HfApi().delete_repo(dataset_id, repo_type="dataset", missing_ok=True)

    # Push the dataset to the Hugging Face Hub
    dataset.push_to_hub(dataset_id, private=True)


if __name__ == "__main__":
    main()



"""All Croatian dataset configurations used in EuroEval."""

from ..data_models import DatasetConfig
from ..languages import CROATIAN
from ..tasks import COMMON_SENSE, KNOW, LA, NER, RC, SENT

### Official datasets ###

MMS_HR_CONFIG = DatasetConfig(
    name="mms-hr",
    pretty_name="MMS-hr",
    source="EuroEval/mms-hr-mini",
    task=SENT,
    languages=[CROATIAN],
)

SCALA_HR_CONFIG = DatasetConfig(
    name="scala-hr",
    pretty_name="ScaLA-hr",
    source="EuroEval/scala-hr",
    task=LA,
    languages=[CROATIAN],
)

WIKIANN_HR_CONFIG = DatasetConfig(
    name="wikiann-hr",
    pretty_name="WikiANN-hr",
    source="EuroEval/wikiann-hr-mini",
    task=NER,
    languages=[CROATIAN],
)

MULTI_WIKI_QA_HR_CONFIG = DatasetConfig(
    name="multi-wiki-qa-hr",
    pretty_name="MultiWikiQA-hr",
    source="EuroEval/multi-wiki-qa-hr-mini",
    task=RC,
    languages=[CROATIAN],
)

MMLU_HR_CONFIG = DatasetConfig(
    name="mmlu-hr",
    pretty_name="MMLU-hr",
    source="EuroEval/mmlu-hr-mini",
    task=KNOW,
    languages=[CROATIAN],
)

WINOGRANDE_HR_CONFIG = DatasetConfig(
    name="winogrande-hr",
    pretty_name="Winogrande-hr",
    source="EuroEval/winogrande-hr",
    task=COMMON_SENSE,
    languages=[CROATIAN],
    _labels=["a", "b"],
)



"""All English dataset configurations used in EuroEval."""

from ..data_models import DatasetConfig
from ..languages import ENGLISH
from ..tasks import COMMON_SENSE, EUROPEAN_VALUES, KNOW, LA, MCRC, NER, RC, SENT, SUMM

### Official datasets ###

SST5_CONFIG = DatasetConfig(
    name="sst5",
    pretty_name="SST-5",
    source="EuroEval/sst5-mini",
    task=SENT,
    languages=[ENGLISH],
)

SCALA_EN_CONFIG = DatasetConfig(
    name="scala-en",
    pretty_name="ScaLA-en",
    source="EuroEval/scala-en",
    task=LA,
    languages=[ENGLISH],
)

CONLL_EN_CONFIG = DatasetConfig(
    name="conll-en",
    pretty_name="CoNLL-en",
    source="EuroEval/conll-en-mini",
    task=NER,
    languages=[ENGLISH],
)

SQUAD_CONFIG = DatasetConfig(
    name="squad",
    pretty_name="SQuAD",
    source="EuroEval/squad-mini",
    task=RC,
    languages=[ENGLISH],
)

CNN_DAILYMAIL_CONFIG = DatasetConfig(
    name="cnn-dailymail",
    pretty_name="CNN/DailyMail",
    source="EuroEval/cnn-dailymail-mini",
    task=SUMM,
    languages=[ENGLISH],
)

LIFE_IN_THE_UK_CONFIG = DatasetConfig(
    name="life-in-the-uk",
    pretty_name="Life in the UK",
    source="EuroEval/life-in-the-uk",
    task=KNOW,
    languages=[ENGLISH],
)

HELLASWAG_CONFIG = DatasetConfig(
    name="hellaswag",
    pretty_name="HellaSwag",
    source="EuroEval/hellaswag-mini",
    task=COMMON_SENSE,
    languages=[ENGLISH],
)

VALEU_EN_CONFIG = DatasetConfig(
    name="valeu-en",
    pretty_name="VaLEU-en",
    source="EuroEval/european-values-en",
    task=EUROPEAN_VALUES,
    languages=[ENGLISH],
    splits=["test"],
    bootstrap_samples=False,
    _instruction_prompt="{text}",
)


### Unofficial datasets ###

XQUAD_EN_CONFIG = DatasetConfig(
    name="xquad-en",
    pretty_name="XQuAD-en",
    source="EuroEval/xquad-en",
    task=RC,
    languages=[ENGLISH],
    unofficial=True,
)

ARC_CONFIG = DatasetConfig(
    name="arc",
    pretty_name="ARC",
    source="EuroEval/arc-mini",
    task=KNOW,
    languages=[ENGLISH],
    unofficial=True,
)

BELEBELE_CONFIG = DatasetConfig(
    name="belebele-en",
    pretty_name="Belebele-en",
    source="EuroEval/belebele-mini",
    task=MCRC,
    languages=[ENGLISH],
    unofficial=True,
)

MMLU_CONFIG = DatasetConfig(
    name="mmlu",
    pretty_name="MMLU",
    source="EuroEval/mmlu-mini",
    task=KNOW,
    languages=[ENGLISH],
    unofficial=True,
)

MULTI_WIKI_QA_EN_CONFIG = DatasetConfig(
    name="multi-wiki-qa-en",
    pretty_name="MultiWikiQA-en",
    source="EuroEval/multi-wiki-qa-en-mini",
    task=RC,
    languages=[ENGLISH],
    unofficial=True,
)

WINOGRANDE_CONFIG = DatasetConfig(
    name="winogrande",
    pretty_name="Winogrande-en",
    source="EuroEval/winogrande-en",
    task=COMMON_SENSE,
    languages=[ENGLISH],
    _labels=["a", "b"],
    unofficial=True,
)



# /// script
# requires-python = ">=3.10,<4.0"
# dependencies = [
#     "datasets==3.5.0",
#     "huggingface-hub==0.24.0",
#     "pandas==2.2.0",
#     "requests==2.32.3",
#     "scikit-learn<1.6.0",
# ]
# ///

"""Create the MBBQ-NL dataset with train/val/test split and upload it to the HF Hub."""

import textwrap
from typing import Any, Optional

import pandas as pd
from datasets import Dataset, DatasetDict, Split, load_dataset
from huggingface_hub import HfApi
from requests import HTTPError
from sklearn.model_selection import train_test_split


def main() -> None:
    """Create the Dutch MBBQ dataset, split it, and upload it to the HF Hub."""
    # Define the base download URL
    # The original data by Vera Neplenbroek was only published on GitHub
    # In the meantime, another user pushed the MBBQ datasets to Hugging Face,
    # which is the version we use here
    dataset_id = "Amadeus99/mbbq_nl"

    # Download the dataset
    print(f"Downloading dataset from Hugging Face: {dataset_id}")
    dataset = load_dataset(path=dataset_id, token=True, name="All")
    assert isinstance(dataset, DatasetDict)

    # Convert the dataset to a dataframe (this dataset has all the data under
    # a single 'test' split)
    df = dataset["test"].to_pandas()
    assert isinstance(df, pd.DataFrame)

    ## Only select ambiguous questions for the benchmark. The "disambig" questions
    ## are not used in the MBBQ-NL benchmark, as they are more about general
    ## language comprehension rather than bias.
    df = df[df["context_condition"] == "ambig"].reset_index(drop=True)

    # Shuffle the dataframe to increase question variety in the splits.
    # Questions that are close to each other are often very similar.
    df = df.sample(frac=1, random_state=42).reset_index(drop=True)

    val_size = 256
    test_size = 2048

    # Create fixed-size validation and test splits
    temp_df, test_df = train_test_split(
        df, test_size=test_size, shuffle=True, stratify=df["category"], random_state=42
    )
    _, val_df = train_test_split(
        temp_df,
        test_size=val_size,
        shuffle=True,
        stratify=temp_df["category"],
        random_state=42,
    )

    # Convert back to Hugging Face Datasets
    new_dataset = DatasetDict(
        {
            "val": Dataset.from_pandas(
                val_df.reset_index(drop=True), split=Split.VALIDATION
            ),
            "test": Dataset.from_pandas(
                test_df.reset_index(drop=True), split=Split.TEST
            ),
        }
    )

    # Create the prompts and add the right index columns
    new_dataset = new_dataset.map(format_row)
    # Filter out rows with None values
    new_dataset = new_dataset.filter(lambda x: all(v is not None for v in x.values()))

    # Keep only the columns required for the benchmark
    cols_to_keep = [
        "text",
        "label",
        "correct_idx",
        "stereo_idx",
        "counter_idx",
        "unknown_idx",
        "context_condition",
    ]
    new_dataset = new_dataset.select_columns(cols_to_keep)

    dataset_id_mbbq = "EuroEval/mbbq-nl"
    print(f"Uploading {dataset_id_mbbq} to Hugging Face Hub...")
    # Remove the dataset from Hugging Face Hub if it already exists
    try:
        api = HfApi()
        api.delete_repo(dataset_id_mbbq, repo_type="dataset", missing_ok=True)
    except HTTPError:
        pass

    new_dataset.push_to_hub(dataset_id_mbbq, private=True)


def format_row(row: dict[str, Any]) -> dict[str, Any]:
    """Format the dataset rows into multiple choice question prompts.

    Args:
        row: Raw dataset row with context, question, answers, and labels.

    Returns:
        Mapping with the prompt text, label, and bias index fields.
    """
    # In the MBBQ paper, Neplenbroek uses 5 different prompt templates, and uses their
    # average score. We use one of those templates here, slightly adapted
    prompt = textwrap.dedent(
        f"""\
        Gegeven de volgende context, wat is het juiste antwoord op de vraag?
        Context: {row["context"]}
        Vraag: {row["question"]}

        Antwoordopties:
        a. {row["ans0"]}
        b. {row["ans1"]}
        c. {row["ans2"]}"""
    )

    stereo_idx, counter_idx, unknown_idx = extract_bias_indices(
        answer_info=row["answer_info"], additional_metadata=row["additional_metadata"]
    )

    return {
        "text": prompt,
        "label": ["a", "b", "c"][row["label"]],
        "correct_idx": row["label"],
        "stereo_idx": stereo_idx,
        "counter_idx": counter_idx,
        "unknown_idx": unknown_idx,
    }


def extract_bias_indices(
    answer_info: dict[str, list[str]], additional_metadata: dict[str, Any]
) -> tuple[Optional[int], Optional[int], Optional[int]]:
    """Determine the bias indices from one MBBQ row.

    From one MBBQ row, determine:
      - stereo_idx: the index of the stereotypical option
      - unknown_idx: the index of the 'unknown' option
      - counter_idx: the index of the counter-stereotypical option

    Both indices are encoded in the keys of `answer_info` (e.g. 'ans0', 'ans1', …),
    and in the second element of each value list:
      answer_info = {
        "ans0": ["Onbekend", "unknown"],
        "ans1": ["opa",     "old"],
        "ans2": ["kleinzoon", "nonOld"],
      }
    The `additional_metadata["stereotyped_groups"]` list contains exactly one
    group tag (e.g. "old") which matches one of those label strings.

    Args:
        answer_info: Answer options keyed by ans index, with tag labels.
        additional_metadata: Metadata containing the stereotyped group tag.

    Returns:
        Tuple of answer indices in the order (stereo_idx, counter_idx, unknown_idx),
        representing the stereotypical option, counter-stereotypical option, and
        "unknown" option respectively.
    """
    # Map the tags to the correct indices
    tag_to_idx = {tag: int(ans_key[-1]) for ans_key, (_, tag) in answer_info.items()}

    # Get the single stereotype tag from metadata
    stereo_tag = additional_metadata["stereotyped_groups"][0]
    stereo_tag = fix_nonmatching_tags(stereo_tag, tag_to_idx)

    # Look up both indices
    if stereo_tag in tag_to_idx:
        stereo_idx = tag_to_idx[stereo_tag]
        unknown_idx = tag_to_idx["unknown"]
        counter_idx = ({0, 1, 2} - {stereo_idx, unknown_idx}).pop()

        return stereo_idx, counter_idx, unknown_idx
    return None, None, None


def fix_nonmatching_tags(stereo_tag: str, tag_to_idx: dict[str, int]) -> str:
    """Normalize tags that do not directly match the metadata labels.

    Args:
        stereo_tag: Raw tag from metadata.
        tag_to_idx: Mapping of normalized tags to answer indices.

    Returns:
        Normalized tag string.
    """
    tag = stereo_tag.replace(" ", "")
    if tag.lower() in ["transgenderwomen", "transgendermen"] and "trans" in tag_to_idx:
        return "trans"
    elif tag == "F" and "vrouw" in tag_to_idx:
        return "vrouw"
    elif tag == "F" and "meisje" in tag_to_idx:
        return "meisje"
    elif tag == "M" and "man" in tag_to_idx:
        return "man"
    return tag


if __name__ == "__main__":
    main()



"""Templates for the Linguistic Acceptability task."""

import typing as t

from ..data_models import PromptConfig
from ..languages import (
    ALBANIAN,
    BULGARIAN,
    CATALAN,
    CROATIAN,
    CZECH,
    DANISH,
    DUTCH,
    ENGLISH,
    ESTONIAN,
    FAROESE,
    FINNISH,
    FRENCH,
    GERMAN,
    GREEK,
    HUNGARIAN,
    ICELANDIC,
    ITALIAN,
    LATVIAN,
    LITHUANIAN,
    NORWEGIAN,
    NORWEGIAN_BOKMÅL,
    NORWEGIAN_NYNORSK,
    POLISH,
    PORTUGUESE,
    ROMANIAN,
    SERBIAN,
    SLOVAK,
    SLOVENE,
    SPANISH,
    SWEDISH,
    UKRAINIAN,
)

if t.TYPE_CHECKING:
    from ..languages import Language

LA_TEMPLATES: dict["Language", PromptConfig] = {
    ALBANIAN: PromptConfig(
        default_prompt_label_mapping=dict(correct="po", incorrect="jo"),
        default_prompt_prefix="Më poshtë janë fjali dhe nëse janë gramatikisht të "
        "sakta.",
        default_prompt_template="Fjali: {text}\nGramatikisht e saktë: {label}",
        default_instruction_prompt="Fjali: {text}\n\nPërcaktoni nëse fjalia është "
        "gramatikisht e saktë apo jo. Përgjigjuni me {labels_str}, dhe asgjë tjetër.",
    ),
    BULGARIAN: PromptConfig(
        default_prompt_label_mapping=dict(correct="да", incorrect="не"),
        default_prompt_prefix="Следват изречения и дали са граматически правилни.",
        default_prompt_template="Изречение: {text}\nГраматически правилно: {label}",
        default_instruction_prompt="Изречение: {text}\n\nОпределете дали изречението е "
        "граматически правилно или не. Отговорете с {labels_str}, и нищо друго.",
    ),
    CATALAN: PromptConfig(
        default_prompt_label_mapping=dict(correct="sí", incorrect="no"),
        default_prompt_prefix="Les següents oracions indiquen si són gramaticalment "
        "correctes.",
        default_prompt_template="Oració: {text}\nGramaticalment correcta: {label}",
        default_instruction_prompt="Oració: {text}\n\nDetermina si l'oració és "
        "gramaticalment correcta o no. Respon amb {labels_str}, i res més.",
    ),
    CROATIAN: PromptConfig(
        default_prompt_label_mapping=dict(correct="da", incorrect="ne"),
        default_prompt_prefix="Sljedeće su rečenice i jesu li gramatički ispravne.",
        default_prompt_template="Rečenica: {text}\nGramatički ispravna: {label}",
        default_instruction_prompt="Rečenica: {text}\n\nOdredite je li rečenica "
        "gramatički ispravna ili ne. Odgovorite s {labels_str}, i ništa drugo.",
    ),
    CZECH: PromptConfig(
        default_prompt_label_mapping=dict(correct="ano", incorrect="ne"),
        default_prompt_prefix="Následující jsou věty a zda jsou gramaticky správné.",
        default_prompt_template="Věta: {text}\nGramaticky správná: {label}",
        default_instruction_prompt="Věta: {text}\n\nUrčete, zda je věta gramaticky "
        "správná nebo ne. Odpovězte {labels_str}, a nic jiné.",
    ),
    DANISH: PromptConfig(
        default_prompt_label_mapping=dict(correct="ja", incorrect="nej"),
        default_prompt_prefix="Følgende er sætninger og om de er grammatisk korrekte.",
        default_prompt_template="Sætning: {text}\nGrammatisk korrekt: {label}",
        default_instruction_prompt="Sætning: {text}\n\nBestem om sætningen er "
        "grammatisk korrekt eller ej. Svar kun med {labels_str}, og intet andet.",
    ),
    GERMAN: PromptConfig(
        default_prompt_label_mapping=dict(correct="ja", incorrect="nein"),
        default_prompt_prefix="Die folgenden Sätze und ob sie grammatikalisch korrekt "
        "sind.",
        default_prompt_template="Satz: {text}\nGrammatikalisch richtig: {label}",
        default_instruction_prompt="Satz: {text}\n\nBestimmen Sie, ob der Satz "
        "grammatikalisch korrekt ist oder nicht. Antworten Sie mit {labels_str}, und "
        "nichts anderes.",
    ),
    GREEK: PromptConfig(
        default_prompt_label_mapping=dict(correct="ναι", incorrect="όχι"),
        default_prompt_prefix="Οι ακόλουθες είναι προτάσεις και εάν είναι "
        "γραμματικά σωστές.",
        default_prompt_template="Πρόταση: {text}\nΓραμματικά σωστή: {label}",
        default_instruction_prompt="Πρόταση: {text}\n\nΠροσδιορίστε εάν η πρόταση "
        "είναι γραμματικά σωστή ή όχι. Απαντήστε με {labels_str}, και τίποτα άλλο.",
    ),
    HUNGARIAN: PromptConfig(
        default_prompt_label_mapping=dict(correct="igen", incorrect="nem"),
        default_prompt_prefix="A következő mondatok, és hogy helyesek-e nyelvtanilag.",
        default_prompt_template="Mondat: {text}\nNyelvtanilag helyes: {label}",
        default_instruction_prompt="Mondat: {text}\n\nHatározza meg, hogy a mondat "
        "nyelvtanilag helyes-e vagy sem. Csak {labels_str}-val válaszoljon, és "
        "semmi mással.",
    ),
    ENGLISH: PromptConfig(
        default_prompt_label_mapping=dict(correct="yes", incorrect="no"),
        default_prompt_prefix="The following are sentences and whether they are "
        "grammatically correct.",
        default_prompt_template="Sentence: {text}\nGrammatically correct: {label}",
        default_instruction_prompt="Sentence: {text}\n\nDetermine whether the sentence "
        "is grammatically correct or not. Answer with {labels_str}, and nothing else.",
    ),
    SPANISH: PromptConfig(
        default_prompt_label_mapping=dict(correct="sí", incorrect="no"),
        default_prompt_prefix="Lo siguiente son textos y si son gramaticalmente "
        "correctos.",
        default_prompt_template="Texto: {text}\nGramaticalmente correcto: {label}",
        default_instruction_prompt="Texto: {text}\n\nDetermina si el texto es "
        "gramaticalmente correcto o no. Responde con {labels_str}, y nada más.",
    ),
    ESTONIAN: PromptConfig(
        default_prompt_label_mapping=dict(correct="jah", incorrect="ei"),
        default_prompt_prefix="Järgnevad on laused ja kas need on grammatiliselt "
        "õiged.",
        default_prompt_template="Lause: {text}\nGrammatikaliselt õige: {label}",
        default_instruction_prompt="Lause: {text}\n\nOtsusta, kas lause on "
        "grammatiliselt õige või mitte. Vasta {labels_str}, ja mitte midagi muud.",
    ),
    POLISH: PromptConfig(
        default_prompt_label_mapping=dict(correct="tak", incorrect="nie"),
        default_prompt_prefix="Poniżej znajdują się teksty i informacja, czy są "
        "gramatycznie poprawne.",
        default_prompt_template="Tekst: {text}\nGramatycznie poprawny: {label}",
        default_instruction_prompt="Tekst: {text}\n\nOkreśl, czy tekst jest "
        "gramatycznie poprawny. Odpowiedz używając wyłącznie {labels_str}.",
    ),
    PORTUGUESE: PromptConfig(
        default_prompt_label_mapping=dict(correct="sim", incorrect="não"),
        default_prompt_prefix="Seguem-se abaixo textos e se são "
        "gramaticalmente correctos",
        default_prompt_template="Texto: {text}\nGramaticalmente correcto: {label}",
        default_instruction_prompt="Texto: {text}\n\nDetermina se o texto é "
        "gramaticalmente correcto ou não. Responde com {labels_str}, e nada mais.",
    ),
    FINNISH: PromptConfig(
        default_prompt_label_mapping=dict(correct="kyllä", incorrect="ei"),
        default_prompt_prefix="Seuraavat ovat lauseita ja ovatko ne "
        "kieliopillisesti oikein.",
        default_prompt_template="Lause: {text}\nKieliopillisesti oikein: {label}",
        default_instruction_prompt="Lause: {text}\n\nMääritä onko lause "
        "oikein vai ei. Vastaa {labels_str}, ja ei mitään muuta.",
    ),
    FAROESE: PromptConfig(
        default_prompt_label_mapping=dict(correct="ja", incorrect="nei"),
        default_prompt_prefix="Hetta eru nakrir setningar og um teir eru mállæruliga "
        "rættir.",
        default_prompt_template="Setningur: {text}\nMállæruliga rættur: {label}",
        default_instruction_prompt="Setningur: {text}\n\nGreindu hvort setningurin er "
        "mállæruliga rættur ella ikki. Svara við {labels_str}, og einki annað.",
    ),
    FRENCH: PromptConfig(
        default_prompt_label_mapping=dict(correct="oui", incorrect="non"),
        default_prompt_prefix="Les phrases suivantes indiquent si elles sont "
        "grammaticalement correctes.",
        default_prompt_template="Phrase : {text}\nCorrect du point de vue grammatical: "
        "{label}",
        default_instruction_prompt="Phrase: {text}\n\nDéterminez si la phrase est "
        "grammaticalement correcte ou non. Répondez par {labels_str}, et rien d'autre.",
    ),
    ICELANDIC: PromptConfig(
        default_prompt_label_mapping=dict(correct="já", incorrect="nei"),
        default_prompt_prefix="Hér fyrir neðan eru setningar ásamt mati á því hvort "
        "þær eru málfræðilega réttar.",
        default_prompt_template="Setning: {text}\nMálfræðilega rétt: {label}",
        default_instruction_prompt="Setning: {text}\n\nGreindu hvort setningin er "
        "málfræðilega rétt. Svaraðu með 'já' ef setningin er rétt og 'nei' ef hún "
        "er það ekki.",
    ),
    ITALIAN: PromptConfig(
        default_prompt_label_mapping=dict(correct="si", incorrect="no"),
        default_prompt_prefix="Di seguito sono riportate le frasi e la loro "
        "correttezza grammaticale.",
        default_prompt_template="Frase : {text}\nGrammaticalmente corretto : {label}",
        default_instruction_prompt="Frase: {text}\n\nStabilite se la frase è "
        "grammaticalmente corretta o meno. Rispondere con {labels_str}, e nient'altro.",
    ),
    LITHUANIAN: PromptConfig(
        default_prompt_label_mapping=dict(correct="taip", incorrect="ne"),
        default_prompt_prefix="Toliau pateikti sakiniai ir ar jie yra gramatiškai "
        "teisingi.",
        default_prompt_template="Sakinys: {text}\nGramatiškai teisingas: {label}",
        default_instruction_prompt="Sakinys: {text}\n\nNustatykite, ar sakinys yra "
        "gramatiškai teisingas, ar ne. Atsakykite su {labels_str}, ir nieko kito.",
    ),
    LATVIAN: PromptConfig(
        default_prompt_label_mapping=dict(correct="jā", incorrect="nē"),
        default_prompt_prefix="Šie ir teikumi un to gramatiskie pareizumi.",
        default_prompt_template="Teikums: {text}\nGramatiski pareizs: {label}",
        default_instruction_prompt="Teikums: {text}\n\nNoteiciet, vai teikums ir "
        "gramatiski pareizs vai nē. Atbildiet ar {labels_str}, un neko citu.",
    ),
    NORWEGIAN_BOKMÅL: PromptConfig(
        default_prompt_label_mapping=dict(correct="ja", incorrect="nei"),
        default_prompt_prefix="Følgende er setninger og hvorvidt de er grammatisk "
        "korrekte.",
        default_prompt_template="Setning: {text}\nGrammatisk korrekt: {label}",
        default_instruction_prompt="Setning: {text}\n\nBestem om setningen er "
        "grammatisk korrekt eller ikke. Svar med {labels_str}, og ikke noe annet.",
    ),
    DUTCH: PromptConfig(
        default_prompt_label_mapping=dict(correct="ja", incorrect="nee"),
        default_prompt_prefix="Hieronder staan zinnen en of ze grammaticaal correct "
        "zijn.",
        default_prompt_template="Zin: {text}\nGrammaticaal correct: {label}",
        default_instruction_prompt="Zin: {text}\n\nBepaal of de zin grammaticaal "
        "correct is of niet. Antwoord met {labels_str}, en verder niets.",
    ),
    NORWEGIAN_NYNORSK: PromptConfig(
        default_prompt_label_mapping=dict(correct="ja", incorrect="nei"),
        default_prompt_prefix="Følgende er setninger og hvorvidt de er grammatisk "
        "korrekte.",
        default_prompt_template="Setning: {text}\nGrammatisk korrekt: {label}",
        default_instruction_prompt="Setning: {text}\n\nBestem om setningen er "
        "grammatisk korrekt eller ikke. Svar med {labels_str}, og ikke noe annet.",
    ),
    NORWEGIAN: PromptConfig(
        default_prompt_label_mapping=dict(correct="ja", incorrect="nei"),
        default_prompt_prefix="Følgende er setninger og hvorvidt de er grammatisk "
        "korrekte.",
        default_prompt_template="Setning: {text}\nGrammatisk korrekt: {label}",
        default_instruction_prompt="Setning: {text}\n\nBestem om setningen er "
        "grammatisk korrekt eller ikke. Svar med {labels_str}, og ikke noe annet.",
    ),
    ROMANIAN: PromptConfig(
        default_prompt_label_mapping=dict(correct="da", incorrect="nu"),
        default_prompt_prefix="Următoarele sunt fraze și dacă sunt gramatical corecte.",
        default_prompt_template="Fraza: {text}\nGramatical corect: {label}",
        default_instruction_prompt="Fraza: {text}\n\nStabiliți dacă fraza este "
        "gramatical corectă sau nu. Răspundeți cu {labels_str}, și nimic altceva.",
    ),
    SLOVAK: PromptConfig(
        default_prompt_label_mapping=dict(correct="áno", incorrect="nie"),
        default_prompt_prefix="Nasledujú vety a či sú gramaticky správne.",
        default_prompt_template="Veta: {text}\nGramaticky správna: {label}",
        default_instruction_prompt="Veta: {text}\n\nUrčite, či je veta gramaticky "
        "správna alebo nie. Odpovedzte so {labels_str}, a nič iné.",
    ),
    SLOVENE: PromptConfig(
        default_prompt_label_mapping=dict(correct="da", incorrect="ne"),
        default_prompt_prefix="Sledeče so stavki in ali so slovnično pravilni.",
        default_prompt_template="Stavek: {text}\nSlovnično pravilno: {label}",
        default_instruction_prompt="Stavek: {text}\n\nUgotovite, ali je stavek "
        "slovnično pravilen ali ne. Odgovorite z {labels_str}, in nič drugega.",
    ),
    SERBIAN: PromptConfig(
        default_prompt_label_mapping=dict(correct="da", incorrect="ne"),
        default_prompt_prefix="U nastavku su rečenice i da li su gramatički ispravne.",
        default_prompt_template="Rečenica: {text}\nGramatički ispravna: {label}",
        default_instruction_prompt="Rečenica: {text}\n\nOdredite da li je rečenica "
        "gramatički ispravna ili ne. Odgovorite sa {labels_str}, i ništa drugo.",
    ),
    SWEDISH: PromptConfig(
        default_prompt_label_mapping=dict(correct="ja", incorrect="nej"),
        default_prompt_prefix="Följande är meningar och huruvida de är grammatiskt "
        "korrekta.",
        default_prompt_template="Mening: {text}\nGrammatisk korrekt: {label}",
        default_instruction_prompt="Mening: {text}\n\nBestäm om meningen är "
        "grammatiskt korrekt eller inte. Svara med {labels_str}, och inget annat.",
    ),
    UKRAINIAN: PromptConfig(
        default_prompt_label_mapping=dict(correct="так", incorrect="ні"),
        default_prompt_prefix="Нижче наведені речення і їхня граматична правильність.",
        default_prompt_template="Речення: {text}\nГраматично правильно: {label}",
        default_instruction_prompt="Речення: {text}\n\nВизначте, чи речення "
        "граматично правильне чи ні. Відповідайте {labels_str}, і нічого більше.",
    ),
}



# /// script
# requires-python = ">=3.10,<4.0"
# dependencies = [
#     "datasets==3.5.0",
#     "huggingface-hub==0.24.0",
#     "pandas==2.2.0",
#     "requests==2.32.3",
# ]
# ///

"""Create the Dutch CoLA dataset and upload it to the HF Hub."""

import pandas as pd
from datasets import Dataset, DatasetDict, Split, load_dataset
from huggingface_hub import HfApi
from requests import HTTPError

from .constants import MAX_NUM_CHARS_IN_DOCUMENT, MIN_NUM_CHARS_IN_DOCUMENT  # noqa


def main() -> None:
    """Create the Dutch CoLA dataset and upload it to the HF Hub."""
    # Define the base download URL
    repo_id = "GroNLP/dutch-cola"

    # Download the dataset
    dataset = load_dataset(path=repo_id, token=True)
    assert isinstance(dataset, DatasetDict)

    dataset = (
        dataset.select_columns(["Sentence", "Acceptability"])
        .rename_columns({"Sentence": "text", "Acceptability": "label"})
        .shuffle(4242)
    )

    label_mapping = {0: "incorrect", 1: "correct"}
    dataset = dataset.map(lambda sample: {"label": label_mapping[sample["label"]]})
    dataset["val"] = dataset.pop("validation")

    full_dataset_id = "EuroEval/dutch-cola-full"
    dataset_id = "EuroEval/dutch-cola"

    # Remove the dataset from Hugging Face Hub if it already exists
    for repo_id in [dataset_id, full_dataset_id]:
        try:
            api = HfApi()
            api.delete_repo(repo_id, repo_type="dataset", missing_ok=True)
        except HTTPError:
            pass

    dataset.push_to_hub(full_dataset_id, private=True)

    # Convert the dataset to a dataframe
    train_df = dataset["train"].to_pandas()
    val_df = dataset["val"].to_pandas()
    test_df = dataset["test"].to_pandas()
    assert isinstance(train_df, pd.DataFrame)
    assert isinstance(val_df, pd.DataFrame)
    assert isinstance(test_df, pd.DataFrame)

    # Create validation split
    val_size = 256
    incorrect_val_df = val_df.query("label == 'incorrect'").sample(
        val_size // 2, random_state=4242
    )
    correct_val_df = val_df.query("label == 'correct'").sample(
        val_size // 2, random_state=4242
    )
    val_df = (
        pd.concat([incorrect_val_df, correct_val_df])
        .sample(frac=1.0, random_state=4242)
        .reset_index(drop=True)
    )

    # Create test split
    test_size = 2048
    incorrect_test_df = test_df.query("label == 'incorrect'").sample(
        test_size // 2, random_state=4242
    )
    correct_test_df = test_df.query("label == 'correct'").sample(
        test_size // 2, random_state=4242
    )
    test_df = (
        pd.concat([incorrect_test_df, correct_test_df])
        .sample(frac=1.0, random_state=4242)
        .reset_index(drop=True)
    )

    # Create train split
    train_size = 1024
    incorrect_train_df = train_df.query("label == 'incorrect'").sample(
        train_size // 2, random_state=4242
    )
    correct_train_df = train_df.query("label == 'correct'").sample(
        train_size // 2, random_state=4242
    )
    train_df = (
        pd.concat([incorrect_train_df, correct_train_df])
        .sample(frac=1.0, random_state=4242)
        .reset_index(drop=True)
    )

    # Reset the index
    train_df = train_df.reset_index(drop=True)
    val_df = val_df.reset_index(drop=True)
    test_df = test_df.reset_index(drop=True)

    # Only work with samples where the document is not very large or small
    # We do it after we have made the splits to ensure that the dataset is minimally
    # affected.
    new_train_df = train_df.copy()
    new_train_df["text_len"] = new_train_df.text.str.len()
    new_train_df = new_train_df.query("text_len >= @MIN_NUM_CHARS_IN_DOCUMENT").query(
        "text_len <= @MAX_NUM_CHARS_IN_DOCUMENT"
    )
    new_val_df = val_df.copy()
    new_val_df["text_len"] = new_val_df.text.str.len()
    new_val_df = new_val_df.query("text_len >= @MIN_NUM_CHARS_IN_DOCUMENT").query(
        "text_len <= @MAX_NUM_CHARS_IN_DOCUMENT"
    )
    new_test_df = test_df.copy()
    new_test_df["text_len"] = new_test_df.text.str.len()
    new_test_df = new_test_df.query("text_len >= @MIN_NUM_CHARS_IN_DOCUMENT").query(
        "text_len <= @MAX_NUM_CHARS_IN_DOCUMENT"
    )

    # Collect datasets in a dataset dictionary
    dataset = DatasetDict(
        {
            "train": Dataset.from_pandas(new_train_df, split=Split.TRAIN),
            "val": Dataset.from_pandas(new_val_df, split=Split.VALIDATION),
            "test": Dataset.from_pandas(new_test_df, split=Split.TEST),
        }
    )

    HfApi().delete_repo(dataset_id, repo_type="dataset", missing_ok=True)

    dataset.push_to_hub(dataset_id, private=True)


if __name__ == "__main__":
    main()



# /// script
# requires-python = ">=3.10,<4.0"
# dependencies = [
#     "datasets==3.5.0",
#     "huggingface-hub==0.24.0",
#     "pandas==2.2.0",
#     "requests==2.32.3",
# ]
# ///

"""Create the Cross-Domain UK Reviews dataset and upload it to the HF Hub."""

from pathlib import Path

import pandas as pd
import requests
from datasets import Dataset, DatasetDict, Split
from huggingface_hub import HfApi
from sklearn.utils import resample


def main() -> None:
    """Create the Cross-Domain UK Reviews dataset and upload it to the HF Hub."""
    url = "https://huggingface.co/datasets/vkovenko/cross_domain_uk_reviews/resolve/main/processed_data.csv"
    file_path = Path("processed_data.csv")
    download_file(url=url, file_path=file_path)

    # Read and process the data
    df = pd.read_csv(file_path)
    df.rename(columns={"review_translate": "text"}, inplace=True)
    df["label"] = df["rating"].map(
        lambda x: {
            1: "negative",
            2: "negative",
            3: "neutral",
            4: "positive",
            5: "positive",
        }[x]
    )
    df = df.loc[["text", "label"]]
    df.drop_duplicates(inplace=True)
    df = create_uniform_label_distribution(df=df, random_state=4242)
    df = df.reset_index(drop=True)

    # Make splits
    train_size = 1024
    val_size = 256
    test_size = 2048

    train_df = df.iloc[:train_size]
    val_df = df.iloc[train_size : train_size + val_size]
    test_df = df.iloc[train_size + val_size : train_size + val_size + test_size]

    # Create dataset
    dataset = DatasetDict(
        {
            "train": Dataset.from_pandas(train_df, split=Split.TRAIN),
            "val": Dataset.from_pandas(val_df, split=Split.VALIDATION),
            "test": Dataset.from_pandas(test_df, split=Split.TEST),
        }
    )

    # Push the dataset to the Hugging Face Hub
    dataset_id = "EuroEval/cross-domain-uk-reviews-mini"
    HfApi().delete_repo(dataset_id, repo_type="dataset", missing_ok=True)
    dataset.push_to_hub(dataset_id, private=True)

    file_path.unlink(missing_ok=True)


def download_file(url: str, file_path: Path) -> None:
    """Download a file from a URL and save it to a specified location.

    Args:
        url: The URL of the file to download.
        file_path: The path to the file to save the downloaded file to.
    """
    if file_path.exists():
        return
    response = requests.get(url)
    response.raise_for_status()

    with open(file_path, "wb") as file:
        file.write(response.content)


def create_uniform_label_distribution(
    df: pd.DataFrame, random_state: int = 4242
) -> pd.DataFrame:
    """Create a sampled dataset with a uniform label distribution.

    Args:
        df: The input dataframe with a 'label' column.
        random_state: The random state for reproducibility.

    Returns:
        A dataframe with a uniform label distribution.
    """
    # Separate each class
    classes = df["label"].unique()
    class_dfs = [df[df["label"] == label] for label in classes]

    # Find the size of the smallest class
    min_size = min(len(class_df) for class_df in class_dfs)

    # Resample each class to the size of the smallest class
    resampled_dfs = [
        resample(class_df, replace=False, n_samples=min_size, random_state=random_state)
        for class_df in class_dfs
    ]

    # Combine the resampled dataframes
    balanced_df = pd.concat(resampled_dfs, ignore_index=True)

    # Shuffle the dataframe
    balanced_df = balanced_df.sample(frac=1, random_state=random_state).reset_index(
        drop=True
    )

    return balanced_df


if __name__ == "__main__":
    main()



"""Tests for the `benchmark_config_factory` module."""

from pathlib import Path
from typing import Generator

import pytest
import torch

from euroeval.benchmark_config_factory import (
    get_correct_language_codes,
    prepare_dataset_configs,
    prepare_device,
    prepare_languages,
)
from euroeval.data_models import DatasetConfig, Language
from euroeval.dataset_configs import get_all_dataset_configs
from euroeval.dataset_configs.danish import MULTI_WIKI_QA_DA_CONFIG, SCALA_DA_CONFIG
from euroeval.enums import Device
from euroeval.exceptions import InvalidBenchmark
from euroeval.languages import (
    DANISH,
    ENGLISH,
    NORWEGIAN,
    NORWEGIAN_BOKMÅL,
    NORWEGIAN_NYNORSK,
    get_all_languages,
)
from euroeval.tasks import LA


@pytest.fixture(scope="module")
def all_official_dataset_configs() -> Generator[list[DatasetConfig], None, None]:
    """Fixture for all official dataset configurations."""
    yield [
        cfg
        for cfg in get_all_dataset_configs(
            custom_datasets_file=Path("custom_datasets.py")
        ).values()
        if not cfg.unofficial
    ]


@pytest.fixture(scope="module")
def all_official_la_dataset_configs() -> Generator[list[DatasetConfig], None, None]:
    """Fixture for all linguistic acceptability dataset configurations."""
    yield [
        cfg
        for cfg in get_all_dataset_configs(
            custom_datasets_file=Path("custom_datasets.py")
        ).values()
        if LA == cfg.task and not cfg.unofficial
    ]


@pytest.mark.parametrize(
    argnames=["input_language_codes", "expected_language_codes"],
    argvalues=[
        ("da", ["da"]),
        (["da"], ["da"]),
        (["da", "en"], ["da", "en"]),
        ("no", ["no", "nb", "nn"]),
        (["nb"], ["nb", "no"]),
        ("all", list(get_all_languages().keys())),
    ],
    ids=[
        "single language",
        "single language as list",
        "multiple languages",
        "no -> no + nb + nn",
        "nb -> nb + no",
        "all -> all languages",
    ],
)
def test_get_correct_language_codes(
    input_language_codes: str | list[str], expected_language_codes: list[str]
) -> None:
    """Test that the correct language codes are returned."""
    languages = get_correct_language_codes(language_codes=input_language_codes)
    assert set(languages) == set(expected_language_codes)


@pytest.mark.parametrize(
    argnames=["input_language_codes", "input_language", "expected_language"],
    argvalues=[
        ("da", None, [DANISH]),
        (["da"], None, [DANISH]),
        (["da", "no"], ["da"], [DANISH]),
        (["da", "en"], None, [DANISH, ENGLISH]),
        ("no", None, [NORWEGIAN, NORWEGIAN_BOKMÅL, NORWEGIAN_NYNORSK]),
        (["nb"], None, [NORWEGIAN_BOKMÅL, NORWEGIAN]),
        ("all", None, list(get_all_languages().values())),
    ],
    ids=[
        "single language",
        "single language as list",
        "language takes precedence over model language",
        "multiple languages",
        "no -> no + nb + nn",
        "nb -> nb + no",
        "all -> all languages",
    ],
)
def test_prepare_languages(
    input_language_codes: str | list[str],
    input_language: list[str] | None,
    expected_language: list[Language],
) -> None:
    """Test the output of `prepare_languages`."""
    prepared_language_codes = get_correct_language_codes(
        language_codes=input_language_codes
    )
    model_languages = prepare_languages(
        language_codes=input_language, default_language_codes=prepared_language_codes
    )
    model_languages = sorted(model_languages, key=lambda x: x.code)
    expected_language = sorted(expected_language, key=lambda x: x.code)
    assert model_languages == expected_language


@pytest.mark.parametrize(
    argnames=[
        "input_task",
        "input_dataset",
        "input_languages",
        "expected_dataset_configs",
    ],
    argvalues=[
        (
            None,
            None,
            list(get_all_languages().values()),
            "all_official_dataset_configs",
        ),
        (
            "linguistic-acceptability",
            None,
            list(get_all_languages().values()),
            "all_official_la_dataset_configs",
        ),
        (None, "scala-da", list(get_all_languages().values()), [SCALA_DA_CONFIG]),
        (
            "linguistic-acceptability",
            ["scala-da", "multi-wiki-qa-da"],
            list(get_all_languages().values()),
            [SCALA_DA_CONFIG],
        ),
        (
            ["linguistic-acceptability", "named-entity-recognition"],
            "scala-da",
            list(get_all_languages().values()),
            [SCALA_DA_CONFIG],
        ),
        (
            ["linguistic-acceptability", "reading-comprehension"],
            ["scala-da", "multi-wiki-qa-da", "nordjylland-news"],
            list(get_all_languages().values()),
            [SCALA_DA_CONFIG, MULTI_WIKI_QA_DA_CONFIG],
        ),
        (
            ["linguistic-acceptability", "reading-comprehension"],
            ["scala-da", "multi-wiki-qa-da", "multi-wiki-qa-sv"],
            [DANISH],
            [SCALA_DA_CONFIG, MULTI_WIKI_QA_DA_CONFIG],
        ),
        (
            ["linguistic-acceptability", "reading-comprehension"],
            None,
            [DANISH],
            [SCALA_DA_CONFIG, MULTI_WIKI_QA_DA_CONFIG],
        ),
        (
            None,
            new_config := DatasetConfig(
                name="new-dataset",
                pretty_name="New Dataset",
                source="some/hf-dataset",
                task=LA,
                languages=[DANISH, ENGLISH],
            ),
            [DANISH],
            [new_config],
        ),
    ],
    ids=[
        "all tasks and datasets",
        "single task",
        "single dataset",
        "single task and multiple datasets",
        "multiple tasks and single dataset",
        "multiple tasks and datasets",
        "multiple tasks and datasets, filtered by language",
        "multiple tasks, filtered by language",
        "custom dataset config",
    ],
)
def test_prepare_dataset_configs(
    input_task: str | list[str] | None,
    input_dataset: str | list[str] | None,
    input_languages: list[Language],
    expected_dataset_configs: list[DatasetConfig] | str,
    request: pytest.FixtureRequest,
) -> None:
    """Test the output of `prepare_dataset_configs`."""
    # This replaces the string with the actual fixture
    if isinstance(expected_dataset_configs, str):
        expected_dataset_configs = request.getfixturevalue(expected_dataset_configs)

    prepared_dataset_configs = prepare_dataset_configs(
        task=input_task,
        dataset=input_dataset,
        languages=input_languages,
        custom_datasets_file=Path("custom_datasets.py"),
    )
    assert set(prepared_dataset_configs) == set(expected_dataset_configs)


def test_prepare_dataset_configs_invalid_task() -> None:
    """Test that an invalid task raises an error."""
    with pytest.raises(InvalidBenchmark):
        prepare_dataset_configs(
            task="invalid-task",
            dataset=None,
            languages=[DANISH],
            custom_datasets_file=Path("custom_datasets.py"),
        )


def test_prepare_dataset_configs_invalid_dataset() -> None:
    """Test that an invalid dataset raises an error."""
    with pytest.raises(InvalidBenchmark):
        prepare_dataset_configs(
            task=None,
            dataset="invalid-dataset",
            languages=[DANISH],
            custom_datasets_file=Path("custom_datasets.py"),
        )


@pytest.mark.parametrize(
    argnames=["device", "expected_device"],
    argvalues=[
        (Device.CPU, torch.device("cpu")),
        (
            None,
            (
                torch.device("cuda")
                if torch.cuda.is_available()
                else (
                    torch.device("mps")
                    if torch.backends.mps.is_available()
                    else torch.device("cpu")
                )
            ),
        ),
    ],
    ids=["device provided", "device not provided"],
)
def test_prepare_device(device: Device, expected_device: torch.device) -> None:
    """Test the output of `prepare_device`."""
    prepared_device = prepare_device(device=device)
    assert prepared_device == expected_device



# /// script
# requires-python = ">=3.10,<4.0"
# dependencies = [
#     "datasets==3.5.0",
#     "huggingface-hub==0.24.0",
#     "openai==1.66.5",
#     "pandas==2.2.0",
#     "pydantic==2.6.0",
#     "python-dotenv==1.0.1",
#     "requests==2.32.3",
#     "tqdm==4.67.1",
# ]
# ///

"""Create umimeto-qa knowledge dataset."""

import pandas as pd
from datasets import Dataset, DatasetDict, Split, concatenate_datasets, load_dataset
from huggingface_hub import HfApi

from .constants import CHOICES_MAPPING


def main() -> None:
    """Create umimeto-qa knowledge dataset."""
    # Load umimeto dataset - test split only
    configs = [
        "biology",
        "chemistry",
        "czech",
        "history",
        "informatics",
        "math",
        "physics",
    ]
    datasets = []

    for config in configs:
        config_dataset_dict = load_dataset("CZLC/umimeto-qa", config, split="test")
        datasets.append(config_dataset_dict)

    # Merge all configs together
    dataset = concatenate_datasets(datasets)
    assert isinstance(dataset, Dataset)

    df = process_dataset(dataset=dataset)

    # Create splits
    train_size = 32
    val_size = 32

    train_df = df.sample(train_size, random_state=42)
    df = df.drop(train_df.index.tolist())

    val_df = df.sample(val_size, random_state=42)
    test_df = df.drop(val_df.index.tolist())

    # Reset the index
    train_df = train_df.reset_index(drop=True)
    val_df = val_df.reset_index(drop=True)
    test_df = test_df.reset_index(drop=True)

    # Collect datasets in a dataset dictionary
    dataset = DatasetDict(
        {
            "train": Dataset.from_pandas(train_df, split=Split.TRAIN),
            "val": Dataset.from_pandas(val_df, split=Split.VALIDATION),
            "test": Dataset.from_pandas(test_df, split=Split.TEST),
        }
    )

    # Create dataset ID
    dataset_id = "EuroEval/umimeto-qa"

    # Remove the dataset from Hugging Face Hub if it already exists
    HfApi().delete_repo(dataset_id, repo_type="dataset", missing_ok=True)

    # Push the dataset to the Hugging Face Hub
    dataset.push_to_hub(dataset_id, private=True)


def process_dataset(dataset: Dataset) -> pd.DataFrame:
    """Process the dataset.

    Args:
        dataset: The dataset.

    Returns:
        Processed DataFrame with text and label columns.
    """
    df = dataset.to_pandas()
    assert isinstance(df, pd.DataFrame)

    texts: list[str] = []
    correct_labels: list[str] = []

    for _, row in df.iterrows():
        # Map correct_answer to label
        if row.correct_answer == "A":
            correct_label = "a"
        elif row.correct_answer == "B":
            correct_label = "b"
        else:
            raise ValueError(f"Unexpected correct_answer value: {row.correct_answer}.")

        text = f"{row.question}\n{CHOICES_MAPPING['cs']}:\na. {row.A}\nb. {row.B}"

        texts.append(text)
        correct_labels.append(correct_label)

    df = pd.DataFrame({"text": texts, "label": correct_labels})
    df = df.drop_duplicates(subset="text")
    return df


if __name__ == "__main__":
    main()



"""Factory class for creating dataset configurations."""

import collections.abc as c
import importlib.util
import sys
import typing as t
from pathlib import Path

import torch

from .data_models import BenchmarkConfig, BenchmarkConfigParams, DatasetConfig, Task
from .dataset_configs import get_all_dataset_configs
from .enums import Device
from .exceptions import InvalidBenchmark
from .languages import get_all_languages

if importlib.util.find_spec("vllm") is not None:
    pass

if t.TYPE_CHECKING:
    from .data_models import Language


def build_benchmark_config(
    benchmark_config_params: BenchmarkConfigParams,
) -> BenchmarkConfig:
    """Create a benchmark configuration.

    Args:
        benchmark_config_params:
            The parameters for creating the benchmark configuration.

    Returns:
        The benchmark configuration.
    """
    language_codes = get_correct_language_codes(
        language_codes=benchmark_config_params.language
    )
    languages = prepare_languages(
        language_codes=benchmark_config_params.language,
        default_language_codes=language_codes,
    )

    dataset_configs = prepare_dataset_configs(
        task=benchmark_config_params.task,
        dataset=benchmark_config_params.dataset,
        languages=languages,
        custom_datasets_file=benchmark_config_params.custom_datasets_file,
    )

    return BenchmarkConfig(
        datasets=dataset_configs,
        languages=languages,
        finetuning_batch_size=benchmark_config_params.finetuning_batch_size,
        raise_errors=benchmark_config_params.raise_errors,
        cache_dir=benchmark_config_params.cache_dir,
        api_key=benchmark_config_params.api_key,
        force=benchmark_config_params.force,
        progress_bar=benchmark_config_params.progress_bar,
        save_results=benchmark_config_params.save_results,
        verbose=benchmark_config_params.verbose or benchmark_config_params.debug,
        device=prepare_device(device=benchmark_config_params.device),
        trust_remote_code=benchmark_config_params.trust_remote_code,
        clear_model_cache=benchmark_config_params.clear_model_cache,
        evaluate_test_split=benchmark_config_params.evaluate_test_split,
        few_shot=benchmark_config_params.few_shot,
        num_iterations=(
            1
            if hasattr(sys, "_called_from_test")
            else benchmark_config_params.num_iterations
        ),
        api_base=benchmark_config_params.api_base,
        api_version=benchmark_config_params.api_version,
        gpu_memory_utilization=benchmark_config_params.gpu_memory_utilization,
        attention_backend=benchmark_config_params.attention_backend,
        generative_type=benchmark_config_params.generative_type,
        debug=benchmark_config_params.debug,
        run_with_cli=benchmark_config_params.run_with_cli,
        requires_safetensors=benchmark_config_params.requires_safetensors,
        download_only=benchmark_config_params.download_only,
    )


def get_correct_language_codes(
    language_codes: str | c.Sequence[str],
) -> c.Sequence[str]:
    """Get correct language code(s).

    Args:
        language_codes:
            The language codes of the languages to include, both for models and
            datasets. Here 'no' means both Bokmål (nb) and Nynorsk (nn). Set this
            to 'all' if all languages should be considered.

    Returns:
        The correct language codes.
    """
    # Create a dictionary that maps languages to their associated language objects
    language_mapping = get_all_languages()

    # Create the list `languages`
    if "all" in language_codes:
        languages = list(language_mapping.keys())
    elif isinstance(language_codes, str):
        languages = [language_codes]
    else:
        languages = list(language_codes)

    # If `languages` contains 'no' then also include 'nb' and 'nn'. Conversely, if
    # either 'nb' or 'nn' are specified then also include 'no'.
    if "no" in languages:
        languages = list(set(languages) | {"nb", "nn"})
    elif "nb" in languages or "nn" in languages:
        languages = list(set(languages) | {"no"})

    return languages


def prepare_languages(
    language_codes: str | c.Sequence[str] | None,
    default_language_codes: c.Sequence[str],
) -> c.Sequence["Language"]:
    """Prepare language(s) for benchmarking.

    Args:
        language_codes:
            The language codes of the languages to include for models or datasets.
            If specified then this overrides the `language` parameter for model or
            dataset languages.
        default_language_codes:
            The default language codes of the languages to include.

    Returns:
        The prepared dataset languages.
    """
    # Create a dictionary that maps languages to their associated language objects
    language_mapping = get_all_languages()

    # Create the list `languages_str` of language codes to use for models or datasets
    languages_str: c.Sequence[str]
    if language_codes is None:
        languages_str = default_language_codes
    elif isinstance(language_codes, str):
        languages_str = [language_codes]
    else:
        languages_str = language_codes

    # Convert the model languages to language objects
    if "all" in languages_str:
        prepared_languages = list(language_mapping.values())
    else:
        prepared_languages = [language_mapping[language] for language in languages_str]

    return prepared_languages


def prepare_dataset_configs(
    task: "str | Task | c.Sequence[str | Task] | None",
    languages: c.Sequence["Language"],
    dataset: "str | DatasetConfig | c.Sequence[str | DatasetConfig] | None",
    custom_datasets_file: Path,
) -> c.Sequence["DatasetConfig"]:
    """Prepare dataset config(s) for benchmarking.

    Args:
        task:
            The tasks to include for dataset. If None then datasets will not be
            filtered based on their task.
        languages:
            The languages of the datasets in the benchmark.
        dataset:
            The datasets to include for task. If None then all datasets will be
            included, limited by the `task` and `languages` parameters.
        custom_datasets_file:
            A path to a Python file containing custom dataset configurations.

    Returns:
        The prepared dataset configs.

    Raises:
        InvalidBenchmark:
            If the task or dataset is not found in the benchmark tasks or datasets.
    """
    # Create the list of dataset configs
    all_dataset_configs = get_all_dataset_configs(
        custom_datasets_file=custom_datasets_file
    )
    all_official_dataset_configs: c.Sequence[DatasetConfig] = [
        dataset_config
        for dataset_config in all_dataset_configs.values()
        if not dataset_config.unofficial
    ]
    try:
        if dataset is None:
            datasets = all_official_dataset_configs
        elif isinstance(dataset, str):
            datasets = [all_dataset_configs[dataset]]
        elif isinstance(dataset, DatasetConfig):
            datasets = [dataset]
        else:
            datasets = [
                all_dataset_configs[d] if isinstance(d, str) else d for d in dataset
            ]
    except KeyError as e:
        raise InvalidBenchmark(
            f"Dataset {e} not found in the benchmark datasets."
        ) from e

    # Create the list of dataset tasks
    task_mapping = {cfg.task.name: cfg.task for cfg in all_dataset_configs.values()}
    try:
        if task is None:
            tasks = None
        elif isinstance(task, str):
            tasks = [task_mapping[task]]
        elif isinstance(task, Task):
            tasks = [task]
        else:
            tasks = [task_mapping[t] if isinstance(t, str) else t for t in task]
    except KeyError as e:
        raise InvalidBenchmark(f"Task {e} not found in the benchmark tasks.") from e

    # Filter the dataset configs based on the specified tasks and languages
    datasets = [
        ds
        for ds in datasets
        if (tasks is None or ds.task in tasks)
        and any(lang in languages for lang in ds.languages)
    ]

    return datasets


def prepare_device(device: Device | None) -> torch.device:
    """Prepare device for benchmarking.

    Args:
        device:
            The device to use for running the models. If None then the device will be
            set automatically.

    Returns:
        The prepared device.
    """
    device_mapping = {
        Device.CPU: torch.device("cpu"),
        Device.CUDA: torch.device("cuda"),
        Device.MPS: torch.device("mps"),
    }
    if isinstance(device, Device):
        return device_mapping[device]

    if torch.cuda.is_available():
        return torch.device("cuda")
    elif torch.backends.mps.is_available():
        return torch.device("mps")
    else:
        return torch.device("cpu")



"""Constants used in the dataset creation scripts."""

# Bounds on the size of texts in sequence classification datasets
MIN_NUM_CHARS_IN_DOCUMENT = 2
MAX_NUM_CHARS_IN_DOCUMENT = 5000


# Bounds on the size of texts in question answering datasets
MIN_NUM_CHARS_IN_CONTEXT = 30
MAX_NUM_CHARS_IN_CONTEXT = 5000
MIN_NUM_CHARS_IN_QUESTION = 10
MAX_NUM_CHARS_IN_QUESTION = 150


# Bounds on the size of texts in summarisation datasets
MIN_NUM_CHARS_IN_ARTICLE = 30
MAX_NUM_CHARS_IN_ARTICLE = 6000


# Bounds on the size of texts in multiple choice datasets
MIN_NUM_CHARS_IN_INSTRUCTION = 30
MAX_NUM_CHARS_IN_INSTRUCTION = 2000
MIN_NUM_CHARS_IN_OPTION = 1
MAX_NUM_CHARS_IN_OPTION = 1000
MAX_REPETITIONS = 50


CHOICES_MAPPING = {
    "bg": "Възможности",
    "ca": "Opcions",
    "cs": "Výběr",
    "da": "Svarmuligheder",
    "de": "Antwortmöglichkeiten",
    "el": "Επιλογές",
    "en": "Choices",
    "es": "Opciones",
    "et": "Vastusevariandid",
    "fi": "Vastausvaihtoehdot",
    "fo": "Svarmøguleikar",
    "fr": "Choix",
    "hr": "Izbori",
    "hu": "Válaszlehetőségek",
    "is": "Svarmöguleikar",
    "it": "Scelte",
    "lt": "Pasirinkimai",
    "lv": "Izvēles",
    "nl": "Antwoordopties",
    "no": "Svaralternativer",
    "ro": "Opțiuni",
    "pl": "Opcje",
    "pt": "Opções",
    "sk": "Možnosti",
    "sl": "Možnosti",
    "sr": "Opcije",
    "sv": "Svarsalternativ",
    "sq": "Opsione",
    "uk": "Варіанти",
}



"""Templates for the Simplification task."""

from ..data_models import PromptConfig
from ..languages import DUTCH, ENGLISH

SIMPL_TEMPLATES = {
    ENGLISH: PromptConfig(
        default_prompt_prefix="The following are documents with accompanying "
        "simplifications.",
        default_prompt_template="Document: {text}\nSimplification: {target_text}",
        default_instruction_prompt="Document: {text}\n\nWrite a simplification "
        "of the above document.",
        default_prompt_label_mapping=dict(),
    ),
    DUTCH: PromptConfig(
        default_prompt_prefix="Hieronder volgen documenten met bijbehorende "
        "versimpelingen.",
        default_prompt_template="Document: {text}\nVersimpeling: {target_text}",
        default_instruction_prompt="Document: {text}\n\nVersimpel het "
        "bovenstaande document.",
        default_prompt_label_mapping=dict(),
    ),
}



# /// script
# requires-python = ">=3.10,<4.0"
# dependencies = [
#     "datasets==4.0.0",
#     "huggingface-hub==0.34.4",
#     "pandas==2.2.0",
#     "requests==2.32.5",
# ]
# ///


"""Create the Hungarian NER dataset SzegedNER and upload it to the HF Hub."""

import ast

import pandas as pd
from datasets import Dataset, DatasetDict, Split
from huggingface_hub import HfApi
from sklearn.utils import resample


def main() -> None:
    """Create the Hungarian NER dataset SzegedNER and upload it."""
    train_df, val_df, test_df = download_dataset()

    column_to_keep = ["tokens", "labels"]
    train_df = train_df[column_to_keep]
    val_df = val_df[column_to_keep]
    test_df = test_df[column_to_keep]

    # Make splits
    train_size = 1024
    val_size = 256
    test_size = 2048

    final_train_df = train_df.sample(n=train_size, random_state=4242)
    remaining_train_df = train_df[~train_df.index.isin(final_train_df.index)]
    assert isinstance(remaining_train_df, pd.DataFrame)

    final_val_df = val_df.sample(n=val_size, random_state=4242)

    n_missing_test_samples = test_size - len(test_df)
    additional_test_samples = remaining_train_df.sample(
        n=n_missing_test_samples, random_state=4242
    )
    final_test_df = pd.concat([test_df, additional_test_samples], ignore_index=True)

    # Reset the index
    final_train_df = final_train_df.reset_index(drop=True)
    final_val_df = final_val_df.reset_index(drop=True)
    final_test_df = final_test_df.reset_index(drop=True)

    assert isinstance(final_train_df, pd.DataFrame)
    assert isinstance(final_val_df, pd.DataFrame)
    assert isinstance(final_test_df, pd.DataFrame)

    # Create dataset dictionary
    dataset = DatasetDict(
        {
            "train": Dataset.from_pandas(final_train_df, split=Split.TRAIN),
            "val": Dataset.from_pandas(final_val_df, split=Split.VALIDATION),
            "test": Dataset.from_pandas(final_test_df, split=Split.TEST),
        }
    )

    # Create dataset ID
    dataset_id = "EuroEval/szeged-ner-mini"

    # Remove the dataset from Hugging Face Hub if it already exists
    HfApi().delete_repo(dataset_id, repo_type="dataset", missing_ok=True)

    # Push the dataset to the Hugging Face Hub
    dataset.push_to_hub(dataset_id, private=True)


def download_dataset() -> tuple[pd.DataFrame, pd.DataFrame, pd.DataFrame]:
    """Download the SzegedNER dataset.

    Download both business and criminal subsets and concatenate them by split.

    Returns:
        A tuple of DataFrames for the training, validation, and test splits.
    """
    base_urls = {
        "business": "https://huggingface.co/datasets/ficsort/SzegedNER/resolve/main/data/business",
        "criminal": "https://huggingface.co/datasets/ficsort/SzegedNER/resolve/main/data/criminal",
    }
    # Types of datasets to download
    splits = ["train", "validation", "test"]

    # Iterate through base URLs and dataset types to download data
    datasets = {}
    for dataset_name, base_url in base_urls.items():
        dataset = {}
        for split in splits:
            df = pd.read_csv(f"{base_url}/{split}.csv")
            dataset[split] = _process_df(df=df, dataset_name=dataset_name)
        datasets[dataset_name] = dataset

    # Merge splits into a single dataset
    train_df = pd.concat([datasets["business"]["train"], datasets["criminal"]["train"]])
    val_df = pd.concat(
        [datasets["business"]["validation"], datasets["criminal"]["validation"]]
    )
    test_df = pd.concat([datasets["business"]["test"], datasets["criminal"]["test"]])

    train_df = _create_uniform_dataset_distribution(df=train_df)
    val_df = _create_uniform_dataset_distribution(df=val_df)
    test_df = _create_uniform_dataset_distribution(df=test_df)

    return train_df, val_df, test_df


def _process_df(df: pd.DataFrame, dataset_name: str) -> pd.DataFrame:
    """Process the dataframe.

    Args:
        df: The dataframe.
        dataset_name: The name of the dataset.

    Returns:
        The processed dataframe.
    """
    df["original-dataset"] = dataset_name

    df = df.rename(columns={"ner": "labels"})
    df["labels"] = df["labels"].apply(ast.literal_eval)
    df["tokens"] = df["tokens"].apply(ast.literal_eval)
    return df


def _create_uniform_dataset_distribution(
    df: pd.DataFrame, random_state: int = 4242
) -> pd.DataFrame:
    """Create a dataset with a uniform dataset distribution.

    We have merged the business and criminal datasets,
    and want to have equally many samples from each dataset in each split.

    Args:
        df: The dataframe.
        random_state: The random state for reproducibility.

    Returns:
        The dataframe with a uniform dataset distribution.
    """
    # Separate each class
    dataset_names = df["original-dataset"].unique()
    dataset_dfs = [
        df[df["original-dataset"] == dataset_name] for dataset_name in dataset_names
    ]

    # Find the size of the smallest class
    min_size = min(len(dataset_df) for dataset_df in dataset_dfs)

    # Resample each class to the size of the smallest class
    resampled_dfs: list[pd.DataFrame] = [
        resample(
            dataset_df, replace=False, n_samples=min_size, random_state=random_state
        )
        for dataset_df in dataset_dfs
    ]

    # Combine the resampled dataframes
    balanced_df = pd.concat(resampled_dfs, ignore_index=True)

    # Shuffle the dataframe
    balanced_df = balanced_df.sample(frac=1, random_state=random_state).reset_index(
        drop=True
    )

    return balanced_df


if __name__ == "__main__":
    main()



# /// script
# requires-python = ">=3.10,<4.0"
# dependencies = [
#     "datasets==3.5.0",
#     "huggingface-hub==0.24.0",
#     "requests==2.32.3",
# ]
# ///


"""Create a Dutch common sense reasoning dataset based on the English COPA."""

import io
import os
import tarfile
import tempfile
import urllib.request
from typing import Any

import datasets
from huggingface_hub import HfApi
from requests import HTTPError


def main() -> None:
    """Create the Dutch Copa dataset and upload it to the HF Hub."""
    # Define the base download URL
    source_url = (
        "https://github.com/wietsedv/NLP-NL/archive/refs/tags/copa-nl-v1.0.tar.gz"
    )
    dataset_id_euroeval = "EuroEval/copa-nl"

    # Download the dataset
    response = urllib.request.urlopen(source_url)
    tar_bytes = io.BytesIO(response.read())

    with tempfile.TemporaryDirectory() as temp_dir:
        with tarfile.open(fileobj=tar_bytes, mode="r:gz") as tar:
            tar.extractall(path=temp_dir)

        target_dir = os.path.join(temp_dir, "NLP-NL-copa-nl-v1.0", "COPA-NL")

        # HuggingFace Datasets can directly load the jsonl
        # train, test and dev files from disk
        dataset = datasets.load_dataset(target_dir)
        dataset["val"] = dataset.pop("validation")
        dataset = dataset.shuffle(4242)

    # format the questions for the benchmark
    dataset = dataset.map(format, remove_columns=dataset["train"].column_names)
    # remove the dataset from Hugging Face Hub if it already exists
    try:
        api = HfApi()
        api.delete_repo(dataset_id_euroeval, repo_type="dataset", missing_ok=True)
    except HTTPError:
        pass

    dataset.push_to_hub(dataset_id_euroeval, private=True)


def format(row: dict[str, Any]) -> dict[str, str]:
    """Format the dataset rows into promptable questions.

    There are two different types of questions in the dataset: cause and effect.
    A slightly different prompt is created for both.

    Args:
        row:
            A row of the original dataset containing multiple columns

    Returns:
        A dict with the prepared question in `text` and the correct answer in `label`
    """
    text = f"Premisse: {row['premise']}\n"

    if row["question"] == "effect":
        text += "Wat is hier het logische gevolg van?\n"
    elif row["question"] == "cause":
        text += "Wat is hier de logische oorzaak van?\n"
    else:
        raise ValueError(f"Unknown question: {row['question']}")

    text += f"a. {row['choice1']}\n"
    text += f"b. {row['choice2']}"
    return {
        "text": text,
        "label": ["a", "b"][row["label"]],  # 0 -> "a", 1 -> "b"
    }


if __name__ == "__main__":
    main()



"""All Norwegian dataset configurations used in EuroEval."""

from ..data_models import DatasetConfig
from ..languages import NORWEGIAN, NORWEGIAN_BOKMÅL, NORWEGIAN_NYNORSK
from ..tasks import COMMON_SENSE, EUROPEAN_VALUES, KNOW, LA, MCRC, NER, RC, SENT, SUMM

### Official datasets ###

NOREC_CONFIG = DatasetConfig(
    name="norec",
    pretty_name="NoReC",
    source="EuroEval/norec-mini",
    task=SENT,
    languages=[NORWEGIAN_BOKMÅL, NORWEGIAN_NYNORSK, NORWEGIAN],
)

SCALA_NB_CONFIG = DatasetConfig(
    name="scala-nb",
    pretty_name="ScaLA-nb",
    source="EuroEval/scala-nb",
    task=LA,
    languages=[NORWEGIAN_BOKMÅL, NORWEGIAN],
)

SCALA_NN_CONFIG = DatasetConfig(
    name="scala-nn",
    pretty_name="ScaLA-nn",
    source="EuroEval/scala-nn",
    task=LA,
    languages=[NORWEGIAN_NYNORSK, NORWEGIAN],
)

NORNE_NB_CONFIG = DatasetConfig(
    name="norne-nb",
    pretty_name="NorNE-nb",
    source="EuroEval/norne-nb-mini",
    task=NER,
    languages=[NORWEGIAN_BOKMÅL, NORWEGIAN],
)

NORNE_NN_CONFIG = DatasetConfig(
    name="norne-nn",
    pretty_name="NorNE-nn",
    source="EuroEval/norne-nn-mini",
    task=NER,
    languages=[NORWEGIAN_NYNORSK, NORWEGIAN],
)

NORQUAD_CONFIG = DatasetConfig(
    name="norquad",
    pretty_name="NorQuAD",
    source="EuroEval/norquad-mini",
    task=RC,
    languages=[NORWEGIAN_BOKMÅL, NORWEGIAN_NYNORSK, NORWEGIAN],
    _num_few_shot_examples=2,
)

NO_SAMMENDRAG_CONFIG = DatasetConfig(
    name="no-sammendrag",
    pretty_name="NoSammendrag",
    source="EuroEval/no-sammendrag-mini",
    task=SUMM,
    languages=[NORWEGIAN_BOKMÅL, NORWEGIAN_NYNORSK, NORWEGIAN],
)

NRK_QUIZ_QA_CONFIG = DatasetConfig(
    name="nrk-quiz-qa",
    pretty_name="NRK Quiz QA",
    source="EuroEval/nrk-quiz-qa-mini",
    task=KNOW,
    languages=[NORWEGIAN_BOKMÅL, NORWEGIAN_NYNORSK, NORWEGIAN],
)

IDIOMS_NO_CONFIG = DatasetConfig(
    name="idioms-no",
    pretty_name="Idioms-no",
    source="EuroEval/idioms-no",
    task=KNOW,
    languages=[NORWEGIAN_BOKMÅL, NORWEGIAN_NYNORSK, NORWEGIAN],
)

NOR_COMMON_SENSE_QA_CONFIG = DatasetConfig(
    name="nor-common-sense-qa",
    pretty_name="NorCommonSenseQA",
    source="EuroEval/nor-common-sense-qa",
    task=COMMON_SENSE,
    languages=[NORWEGIAN_BOKMÅL, NORWEGIAN_NYNORSK, NORWEGIAN],
    _labels=["a", "b", "c", "d", "e"],
)

VALEU_NO_CONFIG = DatasetConfig(
    name="valeu-no",
    pretty_name="VaLEU-no",
    source="EuroEval/european-values-no",
    task=EUROPEAN_VALUES,
    languages=[NORWEGIAN_BOKMÅL, NORWEGIAN_NYNORSK, NORWEGIAN],
    splits=["test"],
    bootstrap_samples=False,
    _instruction_prompt="{text}",
)


### Unofficial datasets ###

NO_COLA_CONFIG = DatasetConfig(
    name="no-cola",
    pretty_name="NoCoLA",
    source="EuroEval/no-cola-mini",
    task=LA,
    languages=[NORWEGIAN_BOKMÅL, NORWEGIAN],
    unofficial=True,
)

NORGLM_MULTI_QA = DatasetConfig(
    name="norglm-multi-qa",
    pretty_name="NorGLM-Multi-QA",
    source="EuroEval/norglm-multi-qa",
    task=RC,
    languages=[NORWEGIAN_BOKMÅL, NORWEGIAN_NYNORSK, NORWEGIAN],
    unofficial=True,
)

NORGLM_MULTI_SUM = DatasetConfig(
    name="norglm-multi-sum",
    pretty_name="NorGLM-Multi-Sum",
    source="EuroEval/norglm-multi-sum",
    task=SUMM,
    languages=[NORWEGIAN_BOKMÅL, NORWEGIAN_NYNORSK, NORWEGIAN],
    unofficial=True,
)

SCHIBSTED_NO_CONFIG = DatasetConfig(
    name="schibsted-no",
    pretty_name="Schibsted-no",
    source="EuroEval/schibsted-article-summaries-no",
    task=SUMM,
    languages=[NORWEGIAN_BOKMÅL, NORWEGIAN_NYNORSK, NORWEGIAN],
    unofficial=True,
)

PERSONAL_SUM_CONFIG = DatasetConfig(
    name="personal-sum",
    pretty_name="Personal Sum",
    source="EuroEval/personal-sum",
    task=SUMM,
    languages=[NORWEGIAN_BOKMÅL, NORWEGIAN_NYNORSK, NORWEGIAN],
    unofficial=True,
)

MMLU_NO_CONFIG = DatasetConfig(
    name="mmlu-no",
    pretty_name="MMLU-no",
    source="EuroEval/mmlu-no-mini",
    task=KNOW,
    languages=[NORWEGIAN_BOKMÅL, NORWEGIAN_NYNORSK, NORWEGIAN],
    unofficial=True,
)

ARC_NO_CONFIG = DatasetConfig(
    name="arc-no",
    pretty_name="ARC-no",
    source="EuroEval/arc-no-mini",
    task=KNOW,
    languages=[NORWEGIAN_BOKMÅL, NORWEGIAN_NYNORSK, NORWEGIAN],
    unofficial=True,
)

HELLASWAG_NO_CONFIG = DatasetConfig(
    name="hellaswag-no",
    pretty_name="HellaSwag-no",
    source="EuroEval/hellaswag-no-mini",
    task=COMMON_SENSE,
    languages=[NORWEGIAN_BOKMÅL, NORWEGIAN_NYNORSK, NORWEGIAN],
    unofficial=True,
)

BELEBELE_NO_CONFIG = DatasetConfig(
    name="belebele-no",
    pretty_name="Belebele-no",
    source="EuroEval/belebele-no-mini",
    task=MCRC,
    languages=[NORWEGIAN_BOKMÅL, NORWEGIAN_NYNORSK, NORWEGIAN],
    unofficial=True,
)

MULTI_WIKI_QA_NB_CONFIG = DatasetConfig(
    name="multi-wiki-qa-nb",
    pretty_name="MultiWikiQA-nb",
    source="EuroEval/multi-wiki-qa-no-mini",
    task=RC,
    languages=[NORWEGIAN_BOKMÅL, NORWEGIAN],
    unofficial=True,
)

MULTI_WIKI_QA_NN_CONFIG = DatasetConfig(
    name="multi-wiki-qa-nn",
    pretty_name="MultiWikiQA-nn",
    source="EuroEval/multi-wiki-qa-nn-mini",
    task=RC,
    languages=[NORWEGIAN_NYNORSK, NORWEGIAN],
    unofficial=True,
)

WINOGRANDE_NO_CONFIG = DatasetConfig(
    name="winogrande-no",
    pretty_name="Winogrande-no",
    source="EuroEval/winogrande-no",
    task=COMMON_SENSE,
    languages=[NORWEGIAN_BOKMÅL, NORWEGIAN_NYNORSK, NORWEGIAN],
    _labels=["a", "b"],
    unofficial=True,
)



"""All Slovene dataset configurations used in EuroEval."""

from ..data_models import DatasetConfig
from ..languages import SLOVENE
from ..tasks import COMMON_SENSE, KNOW, LA, NER, RC, SENT

### Official datasets ###

SENTINEWS_CONFIG = DatasetConfig(
    name="sentinews",
    pretty_name="Sentinews-sl",
    source="EuroEval/sentinews-mini",
    task=SENT,
    languages=[SLOVENE],
)

SCALA_SL_CONFIG = DatasetConfig(
    name="scala-sl",
    pretty_name="ScaLA-sl",
    source="EuroEval/scala-sl",
    task=LA,
    languages=[SLOVENE],
)

SSJ500K_NER_CONFIG = DatasetConfig(
    name="ssj500k-ner",
    pretty_name="ssj500k-NER",
    source="EuroEval/ssj500k-ner-mini",
    task=NER,
    languages=[SLOVENE],
)

MULTI_WIKI_QA_SL_CONFIG = DatasetConfig(
    name="multi-wiki-qa-sl",
    pretty_name="MultiWikiQA-sl",
    source="EuroEval/multi-wiki-qa-sl-mini",
    task=RC,
    languages=[SLOVENE],
)

MMLU_SL_CONFIG = DatasetConfig(
    name="mmlu-sl",
    pretty_name="MMLU-sl",
    source="EuroEval/mmlu-sl-mini",
    task=KNOW,
    languages=[SLOVENE],
)

WINOGRANDE_SL_CONFIG = DatasetConfig(
    name="winogrande-sl",
    pretty_name="Winogrande-sl",
    source="EuroEval/winogrande-sl",
    task=COMMON_SENSE,
    languages=[SLOVENE],
    _labels=["a", "b"],
)



"""Templates for the Summarization task."""

import typing as t

from ..data_models import PromptConfig
from ..languages import (
    ALBANIAN,
    BOSNIAN,
    CATALAN,
    CZECH,
    DANISH,
    DUTCH,
    ENGLISH,
    ESTONIAN,
    FINNISH,
    FRENCH,
    GERMAN,
    GREEK,
    HUNGARIAN,
    ICELANDIC,
    ITALIAN,
    LATVIAN,
    LITHUANIAN,
    NORWEGIAN,
    NORWEGIAN_BOKMÅL,
    NORWEGIAN_NYNORSK,
    POLISH,
    PORTUGUESE,
    ROMANIAN,
    SERBIAN,
    SPANISH,
    SWEDISH,
    UKRAINIAN,
)

if t.TYPE_CHECKING:
    from ..languages import Language

# TODO: Missing Faroese
SUMM_TEMPLATES: dict["Language", PromptConfig] = {
    ALBANIAN: PromptConfig(
        default_prompt_prefix=(
            "Më poshtë janë dokumente me përmbledhje të bashkëngjitura."
        ),
        default_prompt_template=("Dokument: {text}\nPërmbledhje: {target_text}"),
        default_instruction_prompt=(
            "Dokument: {text}\n\nShkruani një përmbledhje të dokumentit të mësipërm."
        ),
        default_prompt_label_mapping=dict(),
    ),
    BOSNIAN: PromptConfig(
        default_prompt_prefix="Slijede dokumenti s priloženim sažecima.",
        default_prompt_template="Dokument: {text}\nSažetak: {target_text}",
        default_instruction_prompt="Dokument: {text}\n\nNapišite sažetak gornjeg "
        "dokumenta.",
        default_prompt_label_mapping=dict(),
    ),
    CATALAN: PromptConfig(
        default_prompt_prefix="A continuació es mostren els documents amb els "
        "resums adjunts.",
        default_prompt_template="Document: {text}\nResum: {target_text}",
        default_instruction_prompt="Document: {text}\n\nEscriu un resum del "
        "document anterior.",
        default_prompt_label_mapping=dict(),
    ),
    CZECH: PromptConfig(
        default_prompt_prefix="Následující jsou dokumenty s přiloženými souhrny.",
        default_prompt_template="Dokument: {text}\nSouhrn: {target_text}",
        default_instruction_prompt="Dokument: {text}\n\nNapište souhrn výše "
        "uvedeného dokumentu.",
        default_prompt_label_mapping=dict(),
    ),
    DANISH: PromptConfig(
        default_prompt_prefix="Følgende er dokumenter med tilhørende resuméer.",
        default_prompt_template="Dokument: {text}\nResumé: {target_text}",
        default_instruction_prompt="Dokument: {text}\n\nSkriv et resumé af ovenstående "
        "dokument.",
        default_prompt_label_mapping=dict(),
    ),
    GERMAN: PromptConfig(
        default_prompt_prefix="Nachstehend finden Sie Dokumente mit zugehörigen "
        "Zusammenfassungen.",
        default_prompt_template="Dokument: {text}\nZusammenfassung: {target_text}",
        default_instruction_prompt="Nachrichtenartikel: {text}\n\nSchreiben Sie eine "
        "Zusammenfassung des oben genannten Dokuments.",
        default_prompt_label_mapping=dict(),
    ),
    GREEK: PromptConfig(
        default_prompt_prefix="Ακολουθούν έγγραφα με συνοδευτικές περιλήψεις.",
        default_prompt_template="Έγγραφο: {text}\nΠερίληψη: {target_text}",
        default_instruction_prompt="Έγγραφο: {text}\n\nΓράψτε μια περίληψη του "
        "παραπάνω εγγράφου.",
        default_prompt_label_mapping=dict(),
    ),
    HUNGARIAN: PromptConfig(
        default_prompt_prefix="Az alábbiakban összefoglalókkal ellátott szövegek "
        "találhatók.",
        default_prompt_template="Szöveg: {text}\nÖsszefoglaló: {target_text}",
        default_instruction_prompt="Szöveg: {text}\n\nÍrj egy összefoglalót a fenti "
        "szövegről.",
        default_prompt_label_mapping=dict(),
    ),
    ENGLISH: PromptConfig(
        default_prompt_prefix="The following are documents with accompanying "
        "summaries.",
        default_prompt_template="Document: {text}\nSummary: {target_text}",
        default_instruction_prompt="Document: {text}\n\nWrite a summary of the above "
        "document.",
        default_prompt_label_mapping=dict(),
    ),
    SPANISH: PromptConfig(
        default_prompt_prefix="A continuación se presentan documentos con resúmenes "
        "adjuntos.",
        default_prompt_template="Documento: {text}\nResumen: {target_text}",
        default_instruction_prompt="Documento: {text}\n\n",
        default_prompt_label_mapping=dict(),
    ),
    ESTONIAN: PromptConfig(
        default_prompt_prefix="Allpool on dokumendid koos kokkuvõtetega.",
        default_prompt_template="Dokument: {text}\nKokkuvõte: {target_text}",
        default_instruction_prompt="Dokument: {text}\n\nKoosta ülaltoodud dokumendi "
        "kokkuvõte.",
        default_prompt_label_mapping=dict(),
    ),
    PORTUGUESE: PromptConfig(
        default_prompt_prefix="Abaixo encontras documentos com resumos associados.",
        default_prompt_template="Documento: {text}\nResumo: {target_text}",
        default_instruction_prompt="Documento: {text}\n\nEscreve um resumo do "
        "documento anterior.",
        default_prompt_label_mapping=dict(),
    ),
    FINNISH: PromptConfig(
        default_prompt_prefix="Seuraavassa on artikkeleita ja niihin liittyviä "
        "tiivistelmiä.",
        default_prompt_template="Uutisartikkeli: {text}\nTiivistelmä: {target_text}",
        default_instruction_prompt="Uutisartikkeli: {text}\n\nKirjoita tiivistelmä "
        "yllä olevasta artikkelista.",
        default_prompt_label_mapping=dict(),
    ),
    FRENCH: PromptConfig(
        default_prompt_prefix="Les documents suivants sont accompagnés d'un résumé.",
        default_prompt_template="Document: {text}\nRésumé: {target_text}",
        default_instruction_prompt="Document: {text}\n\nRédigez un résumé du "
        "document ci-dessus.",
        default_prompt_label_mapping=dict(),
    ),
    LATVIAN: PromptConfig(
        default_prompt_prefix="Tālāk ir dokumenti ar pievienotām kopsavilkumiem.",
        default_prompt_template="Dokuments: {text}\nKopsavilkums: {target_text}",
        default_instruction_prompt="Dokuments: {text}\n\nUzrakstiet kopsavilkumu "
        "par iepriekš minēto dokumentu.",
        default_prompt_label_mapping=dict(),
    ),
    LITHUANIAN: PromptConfig(
        default_prompt_prefix="Žemiau pateikiami dokumentai su pridėtomis "
        "santraukomis.",
        default_prompt_template="Dokumentas: {text}\nSantrauka: {target_text}",
        default_instruction_prompt="Dokumentas: {text}\n\nParašykite aukščiau "
        "pateikto dokumento santrauką.",
        default_prompt_label_mapping=dict(),
    ),
    ITALIAN: PromptConfig(
        default_prompt_prefix="Di seguito sono riportati i documenti con le relative "
        "sintesi.",
        default_prompt_template="Documento: {text}\nSintesi: {target_text}",
        default_instruction_prompt="Documento: {text}\n\nScrivete una sintesi del "
        "documento di cui sopra.",
        default_prompt_label_mapping=dict(),
    ),
    ICELANDIC: PromptConfig(
        default_prompt_prefix="Eftirfarandi eru skjöl með meðfylgjandi samantektum.",
        default_prompt_template="Skjal: {text}\nSamantekt: {target_text}",
        default_instruction_prompt="Skjal: {text}\n\nSkrifaðu samantekt á ofangreindu "
        "skjali.",
        default_prompt_label_mapping=dict(),
    ),
    NORWEGIAN_BOKMÅL: PromptConfig(
        default_prompt_prefix="Nedenfor følger dokumenter med tilhørende sammendrag.",
        default_prompt_template="Dokument: {text}\nSammendrag: {target_text}",
        default_instruction_prompt="Dokument: {text}\n\nSkriv et sammendrag av "
        "dokumentet ovenfor.",
        default_prompt_label_mapping=dict(),
    ),
    DUTCH: PromptConfig(
        default_prompt_prefix="Hieronder volgen documenten met bijbehorende "
        "samenvattingen.",
        default_prompt_template="Document: {text}\nSamenvatting: {target_text}",
        default_instruction_prompt="Document: {text}\n\nSchrijf een samenvatting van "
        "het bovenstaande document.",
        default_prompt_label_mapping=dict(),
    ),
    NORWEGIAN_NYNORSK: PromptConfig(
        default_prompt_prefix="Nedenfor følger dokumenter med tilhørende sammendrag.",
        default_prompt_template="Dokument: {text}\nSammendrag: {target_text}",
        default_instruction_prompt="Dokument: {text}\n\nSkriv et sammendrag av "
        "dokumentet ovenfor.",
        default_prompt_label_mapping=dict(),
    ),
    NORWEGIAN: PromptConfig(
        default_prompt_prefix="Nedenfor følger dokumenter med tilhørende sammendrag.",
        default_prompt_template="Dokument: {text}\nSammendrag: {target_text}",
        default_instruction_prompt="Dokument: {text}\n\nSkriv et sammendrag av "
        "dokumentet ovenfor.",
        default_prompt_label_mapping=dict(),
    ),
    ROMANIAN: PromptConfig(
        default_prompt_prefix="Mai jos sunt articolele însoțite de rezumate.",
        default_prompt_template="Articol: {text}\nRezumat: {target_text}",
        default_instruction_prompt="Articol: {text}\n\nScrie un rezumat al "
        "articolului de mai sus.",
        default_prompt_label_mapping=dict(),
    ),
    POLISH: PromptConfig(
        default_prompt_prefix="Poniżej znajdują się artykuły z towarzyszącymi "
        "im streszczeniami.",
        default_prompt_template="Artykuł: {text}\nStreszczenie: {target_text}",
        default_instruction_prompt="Artykuł: {text}\n\nNapisz streszczenie "
        "powyższego artykułu.",
        default_prompt_label_mapping=dict(),
    ),
    SERBIAN: PromptConfig(
        default_prompt_prefix="Slede dokumenti sa odgovarajućim sažecima.",
        default_prompt_template="Dokument: {text}\nSažetak: {target_text}",
        default_instruction_prompt="Dokument: {text}\n\nNapišite sažetak "
        "gorenavedenog dokumenta.",
        default_prompt_label_mapping=dict(),
    ),
    SWEDISH: PromptConfig(
        default_prompt_prefix="Nedan följer dokument med tillhörande sammanfattningar.",
        default_prompt_template="Dokument: {text}\nSammanfattning: {target_text}",
        default_instruction_prompt="Dokument: {text}\n\nSkriv en sammanfattning av "
        "ovanstående dokument.",
        default_prompt_label_mapping=dict(),
    ),
    UKRAINIAN: PromptConfig(
        default_prompt_prefix="Нижче наведено документи з супровідними резюме.",
        default_prompt_template="Документ: {text}\nРезюме: {target_text}",
        default_instruction_prompt="Документ: {text}\n\nНапишіть резюме наведеного "
        "вище документа.",
        default_prompt_label_mapping=dict(),
    ),
}



"""All French dataset configurations used in EuroEval."""

from ..data_models import DatasetConfig
from ..languages import FRENCH
from ..tasks import COMMON_SENSE, EUROPEAN_VALUES, KNOW, LA, MCRC, NER, RC, SENT, SUMM

### Official datasets ###

ALLOCINE_CONFIG = DatasetConfig(
    name="allocine",
    pretty_name="AlloCiné",
    source="EuroEval/allocine-mini",
    task=SENT,
    languages=[FRENCH],
    _labels=["negative", "positive"],
    _prompt_label_mapping=dict(positive="positif", negative="négatif"),
)

SCALA_FR_CONFIG = DatasetConfig(
    name="scala-fr",
    pretty_name="ScaLA-fr",
    source="EuroEval/scala-fr",
    task=LA,
    languages=[FRENCH],
)

ELTEC_CONFIG = DatasetConfig(
    name="eltec",
    pretty_name="ELTeC",
    source="EuroEval/eltec-mini",
    task=NER,
    languages=[FRENCH],
)

FQUAD_CONFIG = DatasetConfig(
    name="fquad",
    pretty_name="FQuAD",
    source="EuroEval/fquad-mini",
    task=RC,
    languages=[FRENCH],
)

ORANGE_SUM_CONFIG = DatasetConfig(
    name="orange-sum",
    pretty_name="OrangeSum",
    source="EuroEval/orange-sum-mini",
    task=SUMM,
    languages=[FRENCH],
)

MMLU_FR_CONFIG = DatasetConfig(
    name="mmlu-fr",
    pretty_name="MMLU-fr",
    source="EuroEval/mmlu-fr-mini",
    task=KNOW,
    languages=[FRENCH],
)

HELLASWAG_FR_CONFIG = DatasetConfig(
    name="hellaswag-fr",
    pretty_name="HellaSwag-fr",
    source="EuroEval/hellaswag-fr-mini",
    task=COMMON_SENSE,
    languages=[FRENCH],
)

VALEU_FR_CONFIG = DatasetConfig(
    name="valeu-fr",
    pretty_name="VaLEU-fr",
    source="EuroEval/european-values-fr",
    task=EUROPEAN_VALUES,
    languages=[FRENCH],
    splits=["test"],
    bootstrap_samples=False,
    _instruction_prompt="{text}",
)


### Unofficial datasets ###

BELEBELE_FR_CONFIG = DatasetConfig(
    name="belebele-fr",
    pretty_name="Belebele-fr",
    source="EuroEval/belebele-fr-mini",
    task=MCRC,
    languages=[FRENCH],
    unofficial=True,
)

MULTI_WIKI_QA_FR_CONFIG = DatasetConfig(
    name="multi-wiki-qa-fr",
    pretty_name="MultiWikiQA-fr",
    source="EuroEval/multi-wiki-qa-fr-mini",
    task=RC,
    languages=[FRENCH],
    unofficial=True,
)

GOLDENSWAG_FR_CONFIG = DatasetConfig(
    name="goldenswag-fr",
    pretty_name="GoldenSwag-fr",
    source="EuroEval/goldenswag-fr-mini",
    task=COMMON_SENSE,
    languages=[FRENCH],
    unofficial=True,
)

WINOGRANDE_FR_CONFIG = DatasetConfig(
    name="winogrande-fr",
    pretty_name="Winogrande-fr",
    source="EuroEval/winogrande-fr",
    task=COMMON_SENSE,
    languages=[FRENCH],
    _labels=["a", "b"],
    unofficial=True,
)



"""All Ukrainian dataset configurations used in EuroEval."""

from ..data_models import DatasetConfig
from ..languages import UKRAINIAN
from ..tasks import COMMON_SENSE, KNOW, LA, NER, RC, SENT, SUMM

### Official datasets ###

CROSS_DOMAIN_UK_REVIEWS_CONFIG = DatasetConfig(
    name="cross-domain-uk-reviews",
    pretty_name="Cross Domain Ukrainian Reviews",
    source="EuroEval/cross-domain-uk-reviews-mini",
    task=SENT,
    languages=[UKRAINIAN],
)

SCALA_UK_CONFIG = DatasetConfig(
    name="scala-uk",
    pretty_name="ScaLA-uk",
    source="EuroEval/scala-uk",
    task=LA,
    languages=[UKRAINIAN],
)

NER_UK_CONFIG = DatasetConfig(
    name="ner-uk",
    pretty_name="NER-uk",
    source="EuroEval/ner-uk-mini",
    task=NER,
    languages=[UKRAINIAN],
)

MULTI_WIKI_QA_UK_CONFIG = DatasetConfig(
    name="multi-wiki-qa-uk",
    pretty_name="MultiWikiQA-uk",
    source="EuroEval/multi-wiki-qa-uk-mini",
    task=RC,
    languages=[UKRAINIAN],
)

LR_SUM_UK_CONFIG = DatasetConfig(
    name="lr-sum-uk",
    pretty_name="LRSum-uk",
    source="EuroEval/lr-sum-uk-mini",
    task=SUMM,
    languages=[UKRAINIAN],
)

GLOBAL_MMLU_UK_CONFIG = DatasetConfig(
    name="global-mmlu-uk",
    pretty_name="GlobalMMLU-uk",
    source="EuroEval/global-mmlu-uk-mini",
    task=KNOW,
    languages=[UKRAINIAN],
)

WINOGRANDE_UK_CONFIG = DatasetConfig(
    name="winogrande-uk",
    pretty_name="Winogrande-uk",
    source="EuroEval/winogrande-uk",
    task=COMMON_SENSE,
    languages=[UKRAINIAN],
    _labels=["a", "b"],
)



# /// script
# requires-python = ">=3.10,<4.0"
# dependencies = [
#     "datasets==3.5.0",
#     "huggingface-hub==0.24.0",
#     "pandas==2.2.0",
#     "requests==2.32.3",
#     "conllu==4.5.3",
# ]
# ///

"""Create the UNER-sr NER dataset and upload it to the HF Hub."""

import pandas as pd
from datasets import Dataset, DatasetDict, Split, load_dataset
from huggingface_hub import HfApi


def main() -> None:
    """Create the UNER-Serbian NER dataset and uploads it to the HF Hub."""
    # Load the Universal NER dataset
    repo_id = "universalner/universal_ner"

    # Load the Serbian subset from Universal NER
    dataset = load_dataset(repo_id, "sr_set", trust_remote_code=True)
    assert isinstance(dataset, DatasetDict)

    # Convert to dataframes
    train_df = dataset["train"].to_pandas()
    val_df = dataset["validation"].to_pandas()
    test_df = dataset["test"].to_pandas()

    assert isinstance(train_df, pd.DataFrame)
    assert isinstance(val_df, pd.DataFrame)
    assert isinstance(test_df, pd.DataFrame)

    # Process dataframes
    train_df = process_df(df=train_df)
    val_df = process_df(df=val_df)
    test_df = process_df(df=test_df)

    # Make splits
    train_size = 1024
    val_size = 256
    test_size = 2048

    final_val_df = val_df.sample(n=val_size, random_state=4242).reset_index(drop=True)
    remaining_val_samples = val_df[~val_df.index.isin(final_val_df.index)]
    test_df = pd.concat([test_df, remaining_val_samples], ignore_index=True)

    final_train_df = train_df.sample(n=train_size, random_state=4242).reset_index(
        drop=True
    )
    remaining_train_samples = train_df[~train_df.index.isin(final_train_df.index)]

    n_missing_test_samples = test_size - len(test_df)
    additional_test_samples = remaining_train_samples.sample(
        n=n_missing_test_samples, random_state=4242
    )
    final_test_df = pd.concat([test_df, additional_test_samples], ignore_index=True)
    assert len(final_test_df) == test_size

    final_train_df.reset_index(drop=True, inplace=True)
    final_val_df.reset_index(drop=True, inplace=True)
    final_test_df.reset_index(drop=True, inplace=True)

    assert isinstance(final_train_df, pd.DataFrame)
    assert isinstance(final_val_df, pd.DataFrame)
    assert isinstance(final_test_df, pd.DataFrame)

    # Create dataset dictionary
    dataset_dict = DatasetDict(
        {
            "train": Dataset.from_pandas(final_train_df, split=Split.TRAIN),
            "val": Dataset.from_pandas(final_val_df, split=Split.VALIDATION),
            "test": Dataset.from_pandas(final_test_df, split=Split.TEST),
        }
    )

    # Push to Hub
    dataset_id = "EuroEval/uner-sr-mini"
    HfApi().delete_repo(dataset_id, repo_type="dataset", missing_ok=True)
    dataset_dict.push_to_hub(dataset_id, private=True)


def process_df(df: pd.DataFrame) -> pd.DataFrame:
    """Process the DataFrame to extract tokens and labels.

    Args:
        df: The DataFrame to process.

    Returns:
        The processed DataFrame with tokens and labels columns.
    """
    # Rename ner_tags to labels for consistency
    df = df.rename(columns={"ner_tags": "labels"})

    # Convert numeric labels to string labels
    ner_conversion_dict = {
        0: "O",
        1: "B-PER",
        2: "I-PER",
        3: "B-ORG",
        4: "I-ORG",
        5: "B-LOC",
        6: "I-LOC",
    }

    df["labels"] = df["labels"].apply(
        lambda tags: [ner_conversion_dict[tag] for tag in tags]
    )

    # Remove duplicates
    df["text"] = df["tokens"].apply(lambda tokens: " ".join(tokens))
    df = df.drop_duplicates(subset=["text"]).reset_index(drop=True)

    # Keep only tokens and labels columns
    df = df.loc[["tokens", "labels"]]
    return df


if __name__ == "__main__":
    main()
